/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.gridschool.unicore.plugins.povray;

import java.io.File;
import java.util.logging.Level;

import javax.swing.JPanel;

import org.unicore.ajo.ActionGroup;
import org.unicore.ajo.Dependency;
import org.unicore.ajo.IncarnateFiles;
import org.unicore.ajo.Option;
import org.unicore.ajo.UserTask;
import org.unicore.resources.SoftwareResource;
import org.unicore.sets.OptionSet;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.client.panels.IPanelProvider;
import com.pallas.unicore.container.GroupContainer;
import com.pallas.unicore.container.UserContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileExport;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.StringTools;

/**
 * Container building an AJO to execute POV-Ray and manage
 * file imports and exports 
 * 
 *@author     Ralf Ratering
 *@version    $Id: PovrayContainer.java,v 1.1 2004/05/25 14:58:51 rmenday Exp $
 */
public class PovrayContainer extends UserContainer implements IPanelProvider {
	final static long serialVersionUID = -3244691511027502609L;

	private double antiAliasing;

	// povray input that will be executed
	private String sceneDescription;

	private String iniFilename, sceneFilename;

	// make additional outcome panel transient
	private transient RenderPanel outcomePanel;

	private String outputDirectory = ".";

	private int width, height, initialFrame, finalFrame, subsetStartFrame, subsetEndFrame;
	
	private double initialClock, finalClock;

	private boolean exported, animated;

	private File[] outputFiles;

	/**
	 *  Constructor
	 *
	 *@param  parentContainer  parent group container
	 */
	public PovrayContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	private String buildIniString() {
		String iniString = "Input_File_Name=" + getName() + ".pov\n";
		if (isAnimated()) {
			iniString += "Initial_Frame=" + initialFrame + "\n";
			iniString += "Final_Frame=" + finalFrame + "\n";
			iniString += "Subset_Start_Frame=" + subsetStartFrame + "\n";
			iniString += "Subset_End_Frame=" + subsetEndFrame + "\n";
			iniString += "Initial_Clock=" + initialClock + "\n";
			iniString += "Final_Clock=" + finalClock + "\n";
			
		}
		if(antiAliasing>0.0) {
			iniString+="Antialias=On\n";
			iniString += "Antialias_Threshold=" + antiAliasing + "\n";
		}
		else {
			iniString+="Antialias=Off\n";
		}
		iniString += "Width=" + width + "\n";
		iniString += "Height=" + height + "\n";
		iniString += "Output_File_Type=N" + "\n";
		return iniString;
	}

	/**
	 *  Build ActionGroup to transfer and execute the script
	 */
	protected void buildExecuteGroup() {

		//
		// First incarnate editor contents as .pov file
		// and gui parameters as .ini file in uspace
		//
		// get editor contents as 2D byte array
		byte[] sceneContents =
			StringTools.dos2Unix(sceneDescription).getBytes();
		byte[] iniContents = buildIniString().getBytes();

		sceneFilename = getName() + ".pov";
		iniFilename = getName() + ".ini";

		IncarnateFiles incarnateFiles = new IncarnateFiles("IncarnateFiles");
		incarnateFiles.addFile(sceneFilename, sceneContents);
		incarnateFiles.addFile(iniFilename, iniContents);

		//
		// Run PovRay SoftwareResource on ini file
		//

		// pass input filename as environment variable
		OptionSet env = getEnv();
		if (env == null) {
			env = new OptionSet();
			setEnv(env);
		}
		env.add(new Option("POVRAY_INPUTFILE", iniFilename));

		ResourceSet taskResourceSet = getResourceSet().getResourceSetClone();
		taskResourceSet.add(getPreinstalledSoftware());

		UserTask userTask =
			new UserTask(
				"UserTask",
				null,
				taskResourceSet,
				getEnv(),
				getCommandLine(),
				null,
				getRedirectStdout(),
				getRedirectStderr(),
				isVerboseOn(),
				isVersionOn(),
				null,
				getMeasureTime(),
				getDebug(),
				getProfile());

		//
		// Put all tasks into ActionGroup and add dependencies
		//
		executeGroup = new ActionGroup("ExecuteGroup");
		executeGroup.add(incarnateFiles);
		executeGroup.add(userTask);
		try {
			executeGroup.addDependency(
				new Dependency(incarnateFiles, userTask));
		}
		catch (ActionGroup.InvalidDependencyException e) {
			logger.log(Level.SEVERE, "Could not add Dependency", e);
		}
	}

	/**
	 *  Add container specific errors to error set of super classes and select
	 *  appropriate icon for JPA display.
	 *
	 *@return    ErrorSet set containing all errors including those from super
	 *      classes
	 */
	public ErrorSet checkContents() {
		ErrorSet err = super.checkContents();
		// allow empty inputs...
		if (sceneDescription == null) {
			sceneDescription = "";
		}

		if (getVsite() != null) {
			// set POV-Ray as preinstalled software resource if available
			SoftwareResource povrayResource =
				ResourceManager.getResourceSet(
					getVsite()).findSoftwareResourceByName(
					"povray");
			if (povrayResource != null) {
				setPreinstalledSoftware(povrayResource);
			}
			else {
				setPreinstalledSoftware(null);
				err.add(
					new UError(
						id,
						"POV-Ray is not available at " + getVsite().getName()));
			}
		}
		setErrors(err);
		return err;
	}

	/**
	 * This method will be called whenever the plugin task node will be removed
	 * from the job monitor
	 */
	public void finalizePanel() {
		logger.info("Removing plugin task " + getName());
	}

	/**
	 *  Gets the additionalExports attribute of the PovrayContainer object
	 *
	 *@return    The additionalExports value
	 */
	public FileExport[] getAdditionalExports() {
		if (isExported()) {
			if (isAnimated()) {
				int nrOfFrames =  subsetEndFrame-subsetStartFrame + 1;
				if (nrOfFrames < 0) {
					return new FileExport[0];
				}
				FileExport[] exports = new FileExport[nrOfFrames];
				outputFiles = new File[nrOfFrames];
				for (int i = subsetStartFrame; i <= subsetEndFrame; i++) {
					// build export
					String prefix = "";
					if (finalFrame > 9) {
						if (i < 10) {
							prefix = "0";
						}
					}
					else if (finalFrame > 99) {
						if (i < 10) {
							prefix = "00";
						}
						else if (i < 100) {
							prefix = "0";
						}
					}
					String number = prefix + i;

					String outputFilename = getName() + number + ".png";
					String localFilename =
						outputDirectory + File.separator + outputFilename;
					int index = i-subsetStartFrame;
					outputFiles[index] = new File(localFilename);

					exports[index] =
						new FileExport(
							this,
							FileStorage.NSPACE_STRING,
							outputFilename,
							localFilename,
							true,
							false);
				}
				return exports;

			}
			else {
				// build export
				String outputFilename = getName() + ".png";
				String localFilename =
					outputDirectory + File.separator + outputFilename;
				outputFiles = new File[1];
				outputFiles[0]= new File(localFilename);

				FileExport[] exports =
					{
						 new FileExport(
							this,
							FileStorage.NSPACE_STRING,
							outputFilename,
							localFilename,
							true,
							false)};

				return exports;
			}
		}
		return new FileExport[0];
	}
	/**
	 *  Gets the antiAliasing attribute of the PovrayContainer object
	 *
	 *@return    The antiAliasing value
	 */
	public double getAntiAliasing() {
		return antiAliasing;
	}

	/**
	 *  Gets the height attribute of the PovrayContainer object
	 *
	 *@return    The height value
	 */
	public int getHeight() {
		return height;
	}

	/**
	 *  Get the script contents as String
	 *
	 *@return    script contents String
	 */
	public String getSceneDescription() {
		return sceneDescription;
	}

	public void setSceneDescription(String sceneDescription) {
		this.sceneDescription = sceneDescription;
	}

	public int getNrOfPanels() {
		if(isExported()) {
			return 1;
		}
		else {
			return 0;
		}
	}

	public String getOutputDirectory() {
		return outputDirectory;
	}

	public JPanel getPanel(int i) {
		if (outcomePanel == null) {
			outcomePanel = new RenderPanel();
			outcomePanel.setFiles(outputFiles);
		}
		return outcomePanel;
	}

	public String getPanelTitle(int i) {
		return "Image";
	}

	public int getWidth() {
		return width;
	}

	public void setAntiAliasing(double antiAliasing) {
		this.antiAliasing = antiAliasing;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void setOutputDirectory(String outputDirectory) {
		this.outputDirectory = outputDirectory;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public void setInitialFrame(int initialFrame) {
		this.initialFrame = initialFrame;
	}

	public void setFinalFrame(int finalFrame) {
		this.finalFrame = finalFrame;
	}

	public int getInitialFrame() {
		return initialFrame;
	}

	public int getFinalFrame() {
		return finalFrame;
	}
	
	public void setSubsetStartFrame(int subsetStartFrame) {
		this.subsetStartFrame = subsetStartFrame;
	}

	public void setSubsetEndFrame(int subsetEndFrame) {
		this.subsetEndFrame = subsetEndFrame;
	}

	public int getSubsetStartFrame() {
		return subsetStartFrame;
	}

	public int getSubsetEndFrame() {
		return subsetEndFrame;
	}

	public void setInitialClock(double initialClock) {
		this.initialClock = initialClock;
	}

	public void setFinalClock(double finalClock) {
		this.finalClock = finalClock;
	}

	public double getInitialClock() {
		return initialClock;
	}

	public double getFinalClock() {
		return finalClock;
	}
	public void setExported(boolean exported) {
		this.exported = exported;
	}

	public void setAnimated(boolean animated) {
		this.animated = animated;
	}

	public boolean isExported() {
		return exported;
	}
	
	public boolean isAnimated() {
		return animated;
	}
}
