/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.controls;

import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Text field only for int. A beep indicates a wrong input try.
 * 
 * @author Ralf Ratering
 * @version $Id: IntTextField.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class IntTextField extends JTextField {

	private class FieldFocusAdapter extends FocusAdapter {

		private IntTextField field;

		public FieldFocusAdapter(IntTextField field) {
			this.field = field;
		}

		public void focusLost(FocusEvent ev) {
			if (ev.getSource() == field) {
				if (field.getText().equals("")) {
					field.setText("0");
				}
			}
		}
	}

	/**
	 * Minimal main
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		Box panel = new Box(BoxLayout.Y_AXIS);
		panel.add(new IntTextField(-10, 200));
		panel.add(new IntTextField("10", 0, 10000));
		JPanel subPanel = new JPanel();
		subPanel.add(panel);
		JFrame frame = new JFrame();
		frame.getContentPane().add(subPanel);
		frame.setSize(400, 400);
		frame.show();
	}
	private int min, max;

	/**
	 * Constructor sets length of textfield according to min and max value. This
	 * class does not perform a range check!
	 * 
	 * @param min
	 *            minimal value
	 * @param max
	 *            maximal value
	 */
	public IntTextField(int min, int max) {
		this(null, min, max);
	}

	/**
	 * Constructor sets length of textfield according to min and max value and
	 * initializes it with an integer value This class does not perform a range
	 * check!
	 * 
	 * @param start
	 *            initial integer value
	 * @param min
	 *            minimal value
	 * @param max
	 *            maximal value
	 */
	public IntTextField(int start, int min, int max) {
		super(new IntDocument(min, max), new Integer(start).toString(), Math
				.max(("" + min).length(), ("" + max).length()));
		this.min = min;
		this.max = max;
		this.addFocusListener(new FieldFocusAdapter(this));
	}

	/**
	 * Constructor sets length of textfield according to min and max value and
	 * initializes it with a String. This class does not perform a range check!
	 * 
	 * @param text
	 *            String that should contain a integer value
	 * @param min
	 *            minimal value
	 * @param max
	 *            maximal value
	 */
	public IntTextField(String text, int min, int max) {
		super(new IntDocument(min, max), text, Math.max(("" + min).length(),
				("" + max).length()));
		this.min = min;
		this.max = max;
		this.addFocusListener(new FieldFocusAdapter(this));
	}

	/**
	 * Get integer value from textfield
	 * 
	 * @return String from textfield as integer value
	 */
	public int getValue() {
		if (getText().equals("")) {
			setValue(0);
		}
		return Integer.parseInt(getText());
	}

	/**
	 * Set maximal value and adjust textfield length
	 * 
	 * @param max
	 *            new maximal value
	 */
	public void setMaximum(int max) {
		this.max = max;
		// convert min and max to String by adding ""
		setColumns(Math.max(("" + min).length(), ("" + max).length()));
		((IntDocument) getDocument()).setMaximum(max);
	}

	/**
	 * Set minimal value and adjust textfield length
	 * 
	 * @param min
	 *            new minimal value
	 */
	public void setMinimum(int min) {
		this.min = min;
		// convert min and max to String by adding ""
		setColumns(Math.max(("" + min).length(), ("" + max).length()));
		((IntDocument) getDocument()).setMinimum(min);
	}

	/**
	 * Set textfield to an integer value
	 * 
	 * @param value
	 *            new integer value
	 */
	public void setValue(int value) {
		setText((new Integer(value)).toString());
	}
}