/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.dialogs;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.util.ResourceBundle;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;
import com.pallas.unicore.utility.UserMessages;

/**
 * Dialog to enter and verify a new password.
 * 
 * @author Ralf Ratering
 * @version $Id: ChangePasswordDialog.java,v 1.1 2004/05/25 14:58:47 rmenday Exp $
 */
public class ChangePasswordDialog extends GenericDialog {

	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.dialogs.ResourceStrings");
	// text fields
	private JPasswordField oldPasswordField = new JPasswordField(15),
			newPasswordField = new JPasswordField(15),
			verifyPasswordField = new JPasswordField(15);
	private char[] newPassword;
	private char[] oldPassword;

	/**
	 * Constructor builds gui and sets a title
	 * 
	 * @param title
	 *            dialog title
	 * @param parent
	 *            dialog parent - important for modal dialogs
	 */
	public ChangePasswordDialog(JFrame parent, String title) {
		// build generic dialog, assumed to be always modal
		super(parent, title, true, OK_CANCEL);
		buildComponents();
		pack();
	}

	/**
	 * Get the new password from the dialog
	 * 
	 * @return null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getNewPassword() {
		return newPassword;
	}

	/**
	 * Get the old password from the dialog
	 * 
	 * @return null if cancel was pressed and a char array if ok was pressed
	 */
	public char[] getOldPassword() {
		return oldPassword;
	}

	public final static boolean checkEnforcement(char[] password) {
		boolean containsLowercase = false;
		boolean containsUppercase = false;
		boolean containsDigit = false;
		boolean containsPunctuation = false;
		if (password == null || password.length < 8) {
			return false;
		}
		for (int i = 0; i < password.length; i++) {
			if (Character.isDigit(password[i])) {
				containsDigit = true;
			}
			if (Character.isUpperCase(password[i])) {
				containsUppercase = true;
			}
			if (Character.isLowerCase(password[i])) {
				containsLowercase = true;
			}
			if (!Character.isLetterOrDigit(password[i])) {
				containsPunctuation = true;
			}
		}
		return containsDigit && containsUppercase && containsLowercase
				&& containsPunctuation;
	}

	/**
	 * Minimal main
	 * 
	 * @param args
	 *            The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setSize(400, 400);
		frame.show();
		char[] oldPassword = {'H', 'u', 'h', 'u'};
		ChangePasswordDialog dialog = new ChangePasswordDialog(frame,
				"TEST: ChangePasswordDialog");
		dialog.show();
	}

	/**
	 * Put focus into old password field and make okButton default
	 */
	public void show() {
		oldPasswordField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}

	/**
	 * Sets new password to null
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean cancelValues() {
		newPassword = null;
		oldPassword = null;
		return true;
	}

	/**
	 * Takes current textfield entry as new password and compares it to verified
	 * password
	 * 
	 * @return true if dialog can be disposed
	 */
	protected boolean applyValues() {
		// get old password
		oldPassword = oldPasswordField.getPassword();
		// test new password
		newPassword = newPasswordField.getPassword();
		String verifyNewString = new String(verifyPasswordField.getPassword());
		if (!(new String(newPassword)).equals(verifyNewString)) {
			setStatusMessage(res.getString("VERIFICATION_FAILED"));
			return false;
		}
		if (!checkEnforcement(newPassword)) {
			UserMessages.warning(res.getString("WEAK_PASSWORD"));
		}
		// tests passed return ok
		return true;
	}

	/**
	 * Add password fields to the parent dialog
	 */
	private void buildComponents() {
		JLabel oldLabel = new JLabel(res.getString("OLD_PASSWORD"));
		JLabel newLabel = new JLabel(res.getString("NEW_PASSWORD"));
		JLabel verifyLabel = new JLabel(res.getString("VERIFY_PASSWORD"));
		JPanel labelPanel = new JPanel(new GridLayout(3, 1, 20, 20));
		labelPanel.add(oldLabel);
		labelPanel.add(newLabel);
		labelPanel.add(verifyLabel);
		JPanel textPanel = new JPanel(new GridLayout(3, 1, 20, 20));
		textPanel.add(oldPasswordField);
		textPanel.add(newPasswordField);
		textPanel.add(verifyPasswordField);
		JPanel centerPanel = new JPanel();
		centerPanel.add(labelPanel);
		centerPanel.add(textPanel);
		centerPanel.setBorder(new EmptyBorder(20, 5, 10, 5));
		getContentPane().add(centerPanel, BorderLayout.CENTER);
	}
}