/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.Reader;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

/**
 *  A simple text editor with file loading and sacing and undo mechanism
 *
 *@author     Ralf Ratering
 *@version    $Id: TextEditor.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class TextEditor extends JPanel {

	/**
	 * java logger
	 */
	protected static Logger logger =
		Logger.getLogger("com.pallas.unicore.client.editor");
	
	/**
	 * copy text
	 */
	protected CopyAction copyAction;
	
	/**
	 * cut text
	 */
	protected CutAction cutAction;

	/**
	 *  edit menu entries
	 */
	protected JMenuItem cutItem, copyItem, pasteItem, undoItem, redoItem;

	/**
	 *  file to edit
	 */
	protected File file;

	/**
	 * standard file chooser
	 */
	protected JFileChooser fileChooser;

	/**
	 * list of file filters
	 */
	protected Vector fileFilters = new Vector();

	/**
	 * new file
	 */
	protected NewAction newAction;

	/**
	 *  file menu entries
	 */
	protected JMenuItem newItem, openItem, saveItem, saveAsItem;
	
	/**
	 * open file
	 */
	protected OpenAction openAction;

	/**
	 *  paste text
	 */
	protected PasteAction pasteAction;

	/**
	 *  redo
	 */
	protected RedoAction redoAction;

	/**
	 *  save file
	 */
	protected SaveAction saveAction;

	/**
	 *  save file as
	 */
	protected SaveAction saveAsAction;

	/**
	 *  current file filter
	 */
	protected FileFilter selectedFileFilter;

	/**
	 *  edit area
	 */
	protected JTextArea textArea;

	/**
	 *  UndoManager that we add edits to.
	 */
	protected UndoManager undo = new UndoManager();

	/**
	 *  undo
	 */
	protected UndoAction undoAction;

	/**
	 *  Listener for the edits on the current document.
	 */
	protected UndoableEditListener undoHandler = new UndoHandler();

	private JPopupMenu popup;

	private StatusBar statusBar = new StatusBar();

	/**
	 *  Constructor
	 */
	public TextEditor() {
		buildActions();

		buildComponents();

		setStatusMessage("UNICORE TextEditor");
	}

	/**
	 *  add a file filter to file chooser list
	 *
	 *@param  filter  a new file filter
	 */
	public void addFileFilter(FileFilter filter) {
		fileFilters.add(filter);
	}

	/**
	 *  build actions
	 */
	protected void buildActions() {
		newAction = new NewAction();
		openAction = new OpenAction();
		saveAction = new SaveAction();
		saveAsAction = new SaveAsAction();
		cutAction = new CutAction();
		copyAction = new CopyAction();
		pasteAction = new PasteAction();
		undoAction = new UndoAction();
		redoAction = new RedoAction();

	}

	private void buildComponents() {
		// create the embedded JTextComponent
		textArea = new JTextArea();
		setEditorFont(new Font("monospaced", Font.PLAIN, 12));
		// Add this as a listener for undoable edits.
		textArea.getDocument().addUndoableEditListener(undoHandler);
		textArea.addMouseListener(new PopupMouseListener());

		JScrollPane scrollPane = new JScrollPane(textArea);

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEtchedBorder());
		add(buildMenubar(), BorderLayout.NORTH);
		add(scrollPane, BorderLayout.CENTER);
		add(statusBar, BorderLayout.SOUTH);

		buildPopupMenu();
	}

	/**
	 * build menu bar
	 *
	 *@return    menu bar
	 */
	protected JMenuBar buildMenubar() {
		// file menu
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');

		newItem = new JMenuItem(newAction);
		newItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		fileMenu.add(newItem);

		openItem = new JMenuItem(openAction);
		openItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		fileMenu.add(openItem);

		saveItem = new JMenuItem(saveAction);
		saveItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		fileMenu.add(saveItem);

		saveAsItem = new JMenuItem(saveAsAction);
		fileMenu.add(saveAsItem);

		// edit menu
		JMenu editMenu = new JMenu("Edit");
		editMenu.setMnemonic('E');

		cutItem = new JMenuItem(cutAction);
		cutItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		editMenu.add(cutItem);

		copyItem = new JMenuItem(copyAction);
		copyItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		editMenu.add(copyItem);

		pasteItem = new JMenuItem(pasteAction);
		pasteItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
		editMenu.add(pasteItem);

		editMenu.addSeparator();

		undoItem = new JMenuItem(undoAction);
		undoItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK));
		editMenu.add(undoItem);

		redoItem = new JMenuItem(redoAction);
		redoItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK));
		editMenu.add(redoItem);

		JMenuBar menubar = new JMenuBar();
		menubar.add(fileMenu);
		menubar.add(editMenu);

		return menubar;
	}

	private void buildPopupMenu() {
		popup = new JPopupMenu();
		popup.setInvoker(textArea);
		popup.add(cutAction);
		popup.add(copyAction);
		popup.add(pasteAction);
		popup.addSeparator();
		popup.add(undoAction);
		popup.add(redoAction);
	}

	/**
	 *  Fetch the editor contained in this panel
	 *
	 *@return    edit area
	 */
	protected JTextComponent getEditor() {
		return textArea;
	}

	/**
	 *  get editor font
	 *
	 *@return    editor font
	 */
	public Font getEditorFont() {
		return textArea.getFont();
	}

	/**
	 *  get file
	 *
	 *@return    file
	 */
	public File getFile() {
		return file;
	}

	/**
	 *  Find the hosting frame, for the file-chooser dialog.
	 *
	 *@return    parent frame
	 */
	protected Frame getFrame() {
		for (Container p = getParent(); p != null; p = p.getParent()) {
			if (p instanceof Frame) {
				return (Frame) p;
			}
		}
		return null;
	}

	/**
	 *  lien wrap on?
	 *
	 *@return    true, if yes
	 */
	public boolean getLineWrap() {
		return textArea.getLineWrap();
	}

	/**
	 *  get status message
	 *
	 *@return    status message
	 */
	protected String getStatusMessage() {
		return statusBar.getText();
	}

	/**
	 * get current editor contents
	 *
	 *@return    text string
	 */
	public String getText() {
		return textArea.getText();
	}

	private void initFileChooser() {
		// only build the file chooser once...
		if (fileChooser == null) {
			// if we have a filename already use it's directory to start
			if (file != null) {
				if (file.exists()) {
					fileChooser = new JFileChooser(file.getParent());
				}
			}
			fileChooser = new JFileChooser();
			fileChooser.setMultiSelectionEnabled(false);
			fileChooser.setFileHidingEnabled(false);
			for (int i = 0; i < fileFilters.size(); i++) {
				fileChooser.addChoosableFileFilter(
					(FileFilter) fileFilters.elementAt(i));
			}
			if (selectedFileFilter != null) {
				fileChooser.setFileFilter(selectedFileFilter);
			}
		}
	}

	/**
	 *  Helper method initializes menu items
	 *
	 *@param  item   
	 *@param  action 
	 */
	protected void initMenuItem(JMenuItem item, Action action) {
		item.addActionListener(action);
		action.addPropertyChangeListener(new ActionChangedListener(item));
	}

	/**
	 *  Helper method initializing menu item including a CTRL-Shortcut
	 *
	 *@param  item     
	 *@param  keyCode
	 *@param  action  
	 */
	protected void initMenuItem(JMenuItem item, Action action, int keyCode) {
		item.setAccelerator(
			KeyStroke.getKeyStroke(keyCode, ActionEvent.CTRL_MASK));
		initMenuItem(item, action);
	}

	/**
	 *  remove a file filter from file chooser
	 *
	 *@param  filter  file filter
	 */
	public void removeFileFilter(FileFilter filter) {
		fileFilters.remove(filter);
	}

	/**
	 *  Resets the undo manager.
	 */
	protected void resetUndoManager() {
		undo.discardAllEdits();
		undoAction.update();
		redoAction.update();
	}

	/**
	 *  set editor font
	 *
	 *@param  font  editor font
	 */
	public void setEditorFont(Font font) {
		textArea.setFont(font);
	}

	/**
	 * Set the file 
	 *
	 *@param  file  file to edit
	 */
	public void setFile(File file) {
		this.file = file;
		if (file != null && file.exists() && !file.isDirectory()) {
			setStatusMessage("File: " + file.getAbsolutePath());
		}
		else {
			setStatusMessage("New File");
		}
	}

	/**
	 *  Set current file filter
	 *
	 *@param  selectedFileFilter file filter
	 */
	public void setFileFilter(FileFilter selectedFileFilter) {
		this.selectedFileFilter = selectedFileFilter;
	}

	/**
	 *  Line wrapping on?
	 *
	 *@param  wrap  true, if yes
	 */
	public void setLineWrap(boolean wrap) {
		textArea.setLineWrap(wrap);
	}

	/**
	 *  Set status message
	 *
	 *@param  message  status message
	 */
	protected void setStatusMessage(String message) {
		statusBar.setText(message);
	}

	/**
	 * Set editor contents
	 *
	 *@param  text  new text
	 */
	public void setText(String text) {
		textArea.setText(text);
	}

	private void testShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			popup.show(textArea, evt.getX(), evt.getY());
		}
	}

	private class ActionChangedListener implements PropertyChangeListener {
		JMenuItem menuItem;

		ActionChangedListener(JMenuItem mi) {
			super();
			this.menuItem = mi;
		}

		public void propertyChange(PropertyChangeEvent e) {
			String propertyName = e.getPropertyName();
			if (e.getPropertyName().equals(Action.NAME)) {
				String text = (String) e.getNewValue();
				menuItem.setText(text);
			}
			else if (propertyName.equals("enabled")) {
				Boolean enabledState = (Boolean) e.getNewValue();
				menuItem.setEnabled(enabledState.booleanValue());
			}
		}
	}

	class CopyAction extends AbstractAction {
		public CopyAction() {
			super("Copy");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.copy();
		}
	}

	class CutAction extends AbstractAction {
		public CutAction() {
			super("Cut");
		}
		public void actionPerformed(ActionEvent e) {
			textArea.cut();
		}
	}

	class FileLoaderThread extends Thread {

		private File newFile;

		FileLoaderThread(File newFile) {
			this.newFile = newFile;
		}

		public void run() {
			String oldMessage = getStatusMessage();
			try {
				setStatusMessage("Loading file: " + newFile.getAbsolutePath());

				// remove old document
				Document oldDoc = getEditor().getDocument();
				if (oldDoc != null) {
					oldDoc.removeUndoableEditListener(undoHandler);
				}
				getEditor().setDocument(new PlainDocument());

				Document doc = textArea.getDocument();

				// try to start reading
				Reader in = new FileReader(newFile);
				char[] buff = new char[4096];
				int nch;
				while ((nch = in.read(buff, 0, buff.length)) != -1) {
					doc.insertString(
						doc.getLength(),
						new String(buff, 0, nch),
						null);
				}

				// we are done... get rid of progressbar
				doc.addUndoableEditListener(undoHandler);

				resetUndoManager();
				file = newFile;
				setStatusMessage("File: " + file.getAbsolutePath());

			}
			catch (Exception e) {
				setStatusMessage(oldMessage);
			}
		}
	}

	class FileWriterThread extends Thread {

		private Document doc;
		private File file;

		FileWriterThread(File file, Document doc) {
			this.file = file;
			this.doc = doc;
		}

		public void run() {
			try {
				setStatusMessage("Saving file: " + file.getAbsolutePath());
				BufferedWriter bw = new BufferedWriter(new FileWriter(file));
				bw.write(textArea.getText());
				bw.close();
				setStatusMessage(
					"File: " + file.getAbsolutePath() + " successfully saved.");
			}
			catch (Exception e) {
				setStatusMessage(
					"Saving file "
						+ file.getAbsolutePath()
						+ " failed! Reason: "
						+ e.getMessage());
			}

		}
	}

	class NewAction extends AbstractAction {

		NewAction() {
			super("New");
		}

		public void actionPerformed(ActionEvent e) {
			Document oldDoc = getEditor().getDocument();
			if (oldDoc != null) {
				oldDoc.removeUndoableEditListener(undoHandler);
			}
			getEditor().setDocument(new PlainDocument());
			getEditor().getDocument().addUndoableEditListener(undoHandler);
			resetUndoManager();
			file = null;
			setStatusMessage("New File");

			revalidate();
		}
	}

	class OpenAction extends AbstractAction {
		OpenAction() {
			super("Open...");
		}

		public void actionPerformed(ActionEvent e) {
			initFileChooser();

			int result = fileChooser.showOpenDialog(getFrame());
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			File newFile = fileChooser.getSelectedFile();
			if (newFile.exists()) {
				Thread loader = new FileLoaderThread(newFile);
				loader.start();
			}
		}
	}

	class PasteAction extends AbstractAction {
		public PasteAction() {
			super("Paste");
		}

		public void actionPerformed(ActionEvent e) {
			textArea.paste();
		}
	}

	private class PopupMouseListener extends MouseAdapter {
		public void mouseClicked(MouseEvent evt) {
			testShowPopup(evt);
		}

		public void mousePressed(MouseEvent evt) {
			testShowPopup(evt);
		}

		public void mouseReleased(MouseEvent evt) {
			testShowPopup(evt);
		}
	}

	class RedoAction extends AbstractAction {
		public RedoAction() {
			super("Redo");
			setEnabled(false);
		}
		public void actionPerformed(ActionEvent e) {
			try {
				undo.redo();
			}
			catch (CannotRedoException ex) {
				logger.log(Level.WARNING, "Unable to redo.", ex);
			}
			update();
			undoAction.update();
		}

		protected void update() {
			if (undo.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getRedoPresentationName());
			}
			else {
				setEnabled(false);
				putValue(Action.NAME, "Redo");
			}
		}
	}
	class SaveAction extends AbstractAction {

		SaveAction() {
			this("Save");
		}
		SaveAction(String name) {
			super(name);
		}

		public void actionPerformed(ActionEvent e) {
			if (file == null || file.isDirectory()) {
				new SaveAsAction().actionPerformed(e);
				return;
			}
			if (!file.getParentFile().exists()) {
				setStatusMessage(
					"Directory for file "
						+ file.getAbsolutePath()
						+ " does not exist!");
				return;
			}
			save();
		}

		protected void save() {
			Thread saveThread =
				new FileWriterThread(file, textArea.getDocument());
			saveThread.start();
		}
	}

	class SaveAsAction extends SaveAction {
		SaveAsAction() {
			super("Save As...");
		}

		public void actionPerformed(ActionEvent e) {
			initFileChooser();

			int result = fileChooser.showSaveDialog(getFrame());
			if (result != JFileChooser.APPROVE_OPTION) {
				return;
			}

			file = fileChooser.getSelectedFile();

			save();
		}
	}

	class StatusBar extends JPanel {
		private JLabel label = new JLabel();

		public StatusBar() {
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		}

		public String getText() {
			return label.getText();
		}

		public void setText(String text) {
			label.setText(text);
			label.setToolTipText(text);
			removeAll();
			add(label);
			revalidate();
		}
	}

	class UndoAction extends AbstractAction {
		public UndoAction() {
			super("Undo");
			setEnabled(false);
		}

		public void actionPerformed(ActionEvent e) {
			try {
				undo.undo();
			}
			catch (CannotUndoException ex) {
				logger.log(Level.WARNING, "Unable to undo.", ex);
			}
			update();
			redoAction.update();
		}

		protected void update() {
			if (undo.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, undo.getUndoPresentationName());
			}
			else {
				setEnabled(false);
				putValue(Action.NAME, "Undo");
			}
		}
	}

	class UndoHandler implements UndoableEditListener {
		public void undoableEditHappened(UndoableEditEvent e) {
			undo.addEdit(e.getEdit());
			undoAction.update();
			redoAction.update();
		}
	}
	
	/**
	 * a simple test main
	 *
	 *@param  args  The command line arguments
	 */
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.setTitle("Text Editor");
		frame.setBackground(Color.lightGray);
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add("Center", new TextEditor());
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.pack();
		frame.setSize(500, 600);
		frame.show();
	}

}
