/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.explorer;

import java.awt.Component;
import java.io.File;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import com.pallas.unicore.client.tables.AdjustTable;
import com.pallas.unicore.client.tables.DateTableCellRenderer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.VectorSorter;

/**
 * @author Ralf Ratering
 * @version $Id: SelectorTable.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class SelectorTable extends AdjustTable {

	private class FileCellRenderer extends DefaultTableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			JLabel label = (JLabel) super.getTableCellRendererComponent(table,
					value, isSelected, hasFocus, row, column);
			FileEntry fileEntry = (FileEntry) value;
			try {
				Icon icon = view.getSystemIcon(fileEntry.getFile());
				label.setIcon(icon);
			}
			catch (Exception e) {
				logger.log(Level.INFO, "Could not display file icon.", e);
			}
			return label;
		}
	}

	protected class FileTableModel extends DefaultTableModel {

		public boolean isCellEditable(int row, int col) {
			return false;
		}
	}

	private class TableHeaderRenderer extends DefaultTableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			if (column == sortColumn) {
				if (ascending) {
					this.setIcon(up);
				}
				else {
					this.setIcon(down);
				}
			}
			else {
				this.setIcon(null);
			}
			this.setHorizontalTextPosition(JLabel.LEFT);
			JTableHeader header = table.getTableHeader();
			if (header != null) {
				this.setForeground(header.getForeground());
				this.setBackground(header.getBackground());
				this.setFont(header.getFont());
			}
			this.setText((value == null) ? "" : value.toString());
			this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
			return this;
		}
	}
	private final static int COL_DATE = 3;
	private final static int COL_FILE = 0;
	private final static int COL_MEMORY = 1;
	private final static int COL_TYPE = 2;
	private static TableCellRenderer dateCellRenderer = new DateTableCellRenderer(
			ResourceManager.getCompleteFormatter());
	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.explorer");
	private boolean ascending = true;
	private Vector columnIdentifiers;
	private boolean displayIcons;
	private final ImageIcon down = ResourceManager
			.getIcon(ResourceManager.DOWN);
	private FileCellRenderer fileCellRenderer = new FileCellRenderer();
	private FileFilter filter;
	private boolean initialized;
	private int lastSortColumn = -1;
	private File[] listing;
	private FileTableModel model;
	private int sortColumn = 0;
	private final ImageIcon up = ResourceManager.getIcon(ResourceManager.UP);
	protected FileSystemView view;

	public SelectorTable() {
		this(FileSystemView.getFileSystemView(), true);
	}

	public SelectorTable(boolean displayIcons) {
		this(FileSystemView.getFileSystemView(), displayIcons);
	}

	public SelectorTable(FileSystemView view) {
		this(view, true);
	}

	public SelectorTable(FileSystemView view, boolean displayIcons) {
		this.view = view;
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setColumnSelectionAllowed(false);
		setDragEnabled(true);
		buildComponents();
		setDisplayIcons(displayIcons);
		getColumnModel().getColumn(3).setCellRenderer(dateCellRenderer);
		//        ignoreDataWidth(COL_TYPE);
		//        ignoreDataWidth(COL_DATE);
		initialized = false;
	}

	private void buildComponents() {
		columnIdentifiers = new Vector();
		columnIdentifiers.add("Name");
		columnIdentifiers.add("Size");
		columnIdentifiers.add("Type");
		columnIdentifiers.add("Last Modified");
		model = new FileTableModel();
		model.setDataVector(null, columnIdentifiers);
		setModel(model);
		this.getTableHeader().setDefaultRenderer(new TableHeaderRenderer());
	}

	private RowData buildRow(File file) {
		return new RowData(file, view.getSystemDisplayName(file), view
				.getSystemTypeDescription(file));
	}

	private void clear() {
		int rowCount;
		for (int i = 0; i < (rowCount = getRowCount()); i++) {
			model.removeRow(rowCount - 1);
		}
	}

	public File getSelectedFile() {
		int selectedRow = getSelectedRow();
		if (selectedRow < 0) {
			return null;
		}
		RowData rowData = (RowData) model.getDataVector()
				.elementAt(selectedRow);
		return rowData.getFile();
	}

	public void resetValues() {
		this.ascending = true;
		this.sortColumn = 0;
	}

	public void setDisplayIcons(boolean displayIcons) {
		this.displayIcons = displayIcons;
		TableColumn fileColumn = columnModel.getColumn(0);
		if (displayIcons) {
			fileColumn.setCellRenderer(fileCellRenderer);
		}
		else {
			fileColumn.setCellRenderer(null);
		}
	}

	public void setFileFilter(FileFilter filter) {
		FileFilter oldFilter = this.filter;
		this.filter = filter;
		if (oldFilter == null
				|| !oldFilter.getDescription().equals(filter.getDescription())) {
			setListing(listing);
		}
	}

	public void setFileSystemView(FileSystemView view) {
		this.view = view;
	}

	public synchronized void setListing(File[] listing) {
		if (listing == null) {
			return;
		}
		this.listing = listing;
		// remove old rows
		while (model.getRowCount() > 0) {
			model.removeRow(0);
		}
		// add new rows
		for (int i = 0; i < listing.length; i++) {
			if (filter == null || filter.accept(listing[i])) {
				model.addRow(buildRow(listing[i]));
			}
		}
		// find nice column widths
		if (!initialized) {
			initialized = true;
			adjustWidths();
		}
	}

	public void setSelectedObject(File file) {
		if (file == null) {
			selectionModel.clearSelection();
			return;
		}
		Vector dataVector = model.getDataVector();
		for (int i = 0; i < dataVector.size(); i++) {
			RowData rowData = (RowData) dataVector.elementAt(i);
			if (rowData.getFile().equals(file)) {
				selectionModel.setSelectionInterval(i, i);
				return;
			}
		}
	}

	public void sortByColumn(int sortColumn) {
		this.sortColumn = sortColumn;
		if (lastSortColumn == sortColumn) {
			ascending = !ascending;
		}
		else {
			ascending = true;
		}
		RowData.setCompareIndex(sortColumn);
		Vector dataVector = model.getDataVector();
		VectorSorter.sort(dataVector, ascending);
		lastSortColumn = sortColumn;
	}
}