/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.graphs;

import java.util.Vector;

/**
 * Note: this class has a natural ordering that is inconsistent with equals and
 * depends solely on the depth of the vertices in the graph
 * 
 * @author Ralf Ratering
 * @version $Id: DAGVertex.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class DAGVertex extends Vertex implements Comparable {

	private int depth = 0;
	private Vector parents = new Vector(), children = new Vector();

	/**
	 * Constructor for the DAGVertex object
	 * 
	 * @param name
	 *            vertex name
	 */
	public DAGVertex(String name) {
		super(name);
	}

	/**
	 * Adds an edge to the vertex
	 * 
	 * @param edge
	 *            edge that is added to the vertex
	 */
	public void addEdge(Edge edge) {
		DAGVertex parent = (DAGVertex) edge.getParentVertex();
		DAGVertex child = (DAGVertex) edge.getChildVertex();
		if (isOutgoing(edge) && isAncestor(child)) {
			return;
		}
		if (isIncoming(edge) && isDescendant(parent)) {
			return;
		}
		if (getEdges().contains(edge)) {
			return;
		}
		super.addEdge(edge);
		if (child.equals(this) && !parents.contains(parent)) {
			parents.add(parent);
		}
		else if (parent.equals(this) && !children.contains(child)) {
			children.add(child);
		}
	}

	/**
	 * Note: this class has a natural ordering that is inconsistent with equals.
	 * 
	 * @param object
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	public int compareTo(Object object) {
		DAGVertex vertex = (DAGVertex) object;
		if (equals(object)) {
			return 0;
		}
		int value = 1000 * (vertex.getDepth() - getDepth());
		int pathLength = pathLength(this, vertex, 0);
		if (pathLength < 0) {
			pathLength = pathLength(vertex, this, 0);
		}
		if (pathLength > 0) {
			value = value / pathLength;
		}
		return value;
	}

	/**
	 * Gets the Children attribute of the DAGVertex object
	 * 
	 * @return The Children value
	 */
	public Vector getChildren() {
		return children;
	}

	/**
	 * Gets the Depth attribute of the DAGVertex object
	 * 
	 * @return The Depth value
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * Gets the Parents attribute of the DAGVertex object
	 * 
	 * @return The Parents value
	 */
	public Vector getParents() {
		return parents;
	}

	/**
	 * Test if a vertex is an ancestor of the current one
	 * 
	 * @param vertex
	 *            vertex to be tested
	 * @return true if vertex is an ancestor
	 */
	public boolean isAncestor(DAGVertex vertex) {
		for (int i = 0; i < parents.size(); i++) {
			DAGVertex parent = (DAGVertex) parents.elementAt(i);
			if (parent.equals(vertex)) {
				// vertex is parent -> return
				return true;
			}
			// recurse
			if (parent.isAncestor(vertex)) {
				return true;
			}
		}
		// loop finished -> vertex does not match ancestors
		return false;
	}

	/**
	 * Test if a vertex is a descendant of the current one
	 * 
	 * @param vertex
	 *            vertex to be tested
	 * @return true if vertex is a descendant
	 */
	public boolean isDescendant(DAGVertex vertex) {
		for (int i = 0; i < children.size(); i++) {
			DAGVertex child = (DAGVertex) children.elementAt(i);
			if (child.equals(vertex)) {
				// vertex is descendant -> return
				return true;
			}
			// recurse
			if (child.isDescendant(vertex)) {
				return true;
			}
		}
		// loop finished -> vertex does not match descendants
		return false;
	}

	/**
	 * Test if an edge is an incoming one
	 * 
	 * @param edge
	 *            edge to be tested
	 * @return The Incoming value
	 */
	public boolean isIncoming(Edge edge) {
		return edge.getChildVertex().equals(this);
	}

	/**
	 * Test if an edge is an outgoing one.
	 * 
	 * @param edge
	 *            edge to be tested
	 * @return The Outgoing value
	 */
	public boolean isOutgoing(Edge edge) {
		return edge.getParentVertex().equals(this);
	}

	/**
	 * Gets the Root attribute of the DAGVertex object
	 * 
	 * @return The Root value
	 */
	public boolean isRoot() {
		return (depth == 0);
	}

	/**
	 * Description of the Method
	 * 
	 * @param start
	 *            Description of the Parameter
	 * @param end
	 *            Description of the Parameter
	 * @param counter
	 *            Description of the Parameter
	 * @return Description of the Return Value
	 */
	private int pathLength(DAGVertex start, DAGVertex end, int counter) {
		if (start.equals(end)) {
			return counter;
		}
		counter++;
		Vector children = start.getChildren();
		for (int i = 0; i < children.size(); i++) {
			DAGVertex child = (DAGVertex) children.elementAt(i);
			int result = pathLength(child, end, counter);
			if (result > -1) {
				return result;
			}
		}
		return -1;
	}

	/**
	 * Remove an edge from the vertex
	 * 
	 * @param edge
	 *            edge to be removed
	 */
	public void removeEdge(Edge edge) {
		super.removeEdge(edge);
		if (isOutgoing(edge)) {
			children.remove(edge.getChildVertex());
		}
		if (isIncoming(edge)) {
			parents.remove(edge.getParentVertex());
		}
	}

	/**
	 * Set vertex depth in the DAG. The vertex does not compute its depth
	 * itself, because it is easier to compute vertex depths for the whole DAG
	 * at once.
	 * 
	 * @param depth
	 *            The new Depth value
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}
}