/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.panels;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JPanel;

/**
 * Panel displaying an Image
 * 
 * @author Ralf Ratering
 * @version $Id: ImagePanel.java,v 1.1 2004/05/25 14:58:48 rmenday Exp $
 */
public class ImagePanel extends JPanel {

	/**
	 * java logger
	 */
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.panels");
	final static long serialVersionUID = 1384825511689110745L;
	// image width in relation to image height
	private double aspectRatio;
	// image to be displayed
	private Image image;
	// image height
	private int imgHeight;
	// image width
	private int imgWidth;
	// fit image to panel or not
	private boolean resizeOn = true;
	// use tracker to wait for image loading
	private MediaTracker tracker;

	/**
	 * Constructor
	 * 
	 * @param resizeOn
	 *            true, if image will be resized to fit window
	 */
	public ImagePanel(boolean resizeOn) {
		this.resizeOn = resizeOn;
		tracker = new MediaTracker(this);
		setBackground(Color.white);
	}

	/**
	 * Constructor taking image as input
	 * 
	 * @param image
	 *            Image to be displayed
	 * @param resizeOn
	 *            true, if image will be resized to fit window
	 */
	public ImagePanel(Image image, boolean resizeOn) {
		this(resizeOn);
		setImage(image);
	}

	/**
	 * Draw image centered into panel
	 * 
	 * @param g
	 *            graphics context
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (image == null) {
			return;
		}
		Dimension d = getSize();
		int width = d.width;
		int height = d.height;
		int centerX = (int) (width / 2.0);
		int centerY = (int) (height / 2.0);
		// shrink logo to panel size
		if (resizeOn) {
			g.drawImage(image, centerX - width / 2, centerY
					- (int) (width / aspectRatio) / 2, width,
					(int) (width / aspectRatio), this);
		}
		else {
			g.drawImage(image, centerX - imgWidth / 2, centerY - imgHeight / 2,
					this);
		}
	}

	/**
	 * Sets image to be displayed and waits for image to be loaded
	 * 
	 * @param image
	 *            an image
	 */
	public void setImage(Image image) {
		// wait for image to be loaded
		tracker.addImage(image, 0);
		try {
			tracker.waitForID(0);
		}
		catch (InterruptedException e) {
			logger.log(Level.SEVERE, "Tracker interrupted.", e);
		}
		this.image = image;
		imgWidth = image.getWidth(this);
		imgHeight = image.getHeight(this);
		aspectRatio = imgWidth / imgHeight;
		if (!resizeOn) {
			setPreferredSize(new Dimension(imgWidth, imgHeight));
		}
	}

	/**
	 * Set flag if image is resized to panel size
	 * 
	 * @param resizeOn
	 *            true, if image should be resized
	 */
	public void setResizeOn(boolean resizeOn) {
		this.resizeOn = resizeOn;
		repaint();
	}
}