/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.plugins.command;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;
import com.pallas.unicore.client.dialogs.GenericDialog;
import com.pallas.unicore.client.explorer.SelectorDialog;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;

/**
 * User interface to change command default settings
 * 
 * @author Thomas Kentemich
 * @version $Id: CommandDefaultsDialog.java,v 1.1 2004/05/25 14:58:53 rmenday Exp $
 */
public class CommandDefaultsDialog extends GenericDialog {

	private class ButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if (source == commandDirButton) {
				String value = chooseDirectory(res
						.getString("CHOOSE_DIR_TITLE"), commandDefaults
						.getCommandDirectory());
				if (value != null) {
					commandDirTextField.setText(value);
				}
			}
		}
	}
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.plugins.command.ResourceStrings");
	private CommandDefaults commandDefaults;
	private JButton commandDirButton;
	private JTextField commandDirTextField;

	/**
	 * Constructor takes parent frame as input and builds non-modal dialog
	 *  
	 */
	public CommandDefaultsDialog(JFrame parentFrame,
			CommandDefaults commandDefaults) {
		super(parentFrame, res.getString("DEFAULTS_TITLE"), false, OK_CANCEL);
		this.commandDefaults = commandDefaults;
		buildComponents();
		fillTextFields();
		setStatusMessage(res.getString("ENTER_VALUES"));
		pack();
	}

	/**
	 * Add tooltips to buttons from super class
	 */
	private void addTooltipsToButtons() {
		okButton.setToolTipText(res.getString("STORE_CURRENT"));
		applyButton.setToolTipText(res.getString("STORE_AND_CLOSE"));
		cancelButton.setToolTipText(res.getString("CLOSE"));
	}

	private void addToPanel(JPanel panel, Component l, Component r,
			GridBagLayout g, GridBagConstraints c) {
		c.gridwidth = GridBagConstraints.RELATIVE;
		g.setConstraints(l, c);
		panel.add(l);
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(r, c);
		panel.add(r);
	}

	/**
	 * Method will be called from parent class
	 * 
	 * @return true if applying was successful
	 */
	protected boolean applyValues() {
		// copy text field entries to user defaults
		commandDefaults.setCommandDirectory(commandDirTextField.getText()
				.trim());
		// write defaults to file
		if (!commandDefaults.writeToFile()) {
			UserMessages.error(res.getString("COULD_NOT_WRITE"));
			setStatusMessage(res.getString("COULD_NOT_WRITE"));
		}
		else {
			setStatusMessage(res.getString("SUCCESS"));
		}
		// returning true closes dialog if ok was pressed
		return true;
	}

	/**
	 * Build user panel
	 */
	private void buildComponents() {
		// fill text fields with previous defaults
		commandDirTextField = new JTextField(25);
		// file chooser buttons
		commandDirButton = new JButton(ResourceManager
				.getIcon(ResourceManager.OPEN));
		commandDirButton.addActionListener(new ButtonListener());
		JPanel commandDirPanel = new JPanel();
		commandDirPanel.add(commandDirTextField);
		commandDirPanel.add(commandDirButton);
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		JPanel subPanel = new JPanel(gridbag);
		addToPanel(subPanel, new JLabel(res.getString("DIR_LABEL")),
				commandDirPanel, gridbag, c);
		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(subPanel, BorderLayout.CENTER);
		mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		addTooltipsToButtons();
	}

	/**
	 * When cancel was pressed reload user defaults from file
	 * 
	 * @return if canceling was successful
	 */
	protected boolean cancelValues() {
		commandDefaults.loadFromFile();
		fillTextFields();
		return true;
	}

	private String chooseDirectory(String title, String initialdir) {
		SelectorDialog chooser = ResourceManager.getSelectorDialog(new File(
				initialdir), null);
		chooser.setTitle(title);
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int result = chooser.showDialog();
		if (result == JFileChooser.APPROVE_OPTION) {
			try {
				return chooser.getSelectedFile().getCanonicalPath();
			}
			catch (IOException e) {
				return null;
			}
		}
		return null;
	}

	/**
	 * Fill text fields with current class member values
	 */
	private void fillTextFields() {
		commandDirTextField.setText(commandDefaults.getCommandDirectory());
	}

	/**
	 * Overwrite show method to let email text field always grab focus.
	 */
	public void show() {
		commandDirTextField.requestFocus();
		getRootPane().setDefaultButton(okButton);
		super.show();
	}
}