/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.remotefilechooser;

import java.io.File;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.unicore.Vsite;
import org.unicore.outcome.XFile;
import org.unicore.resources.AlternativeUspace;
import org.unicore.resources.Home;
import org.unicore.resources.Storage;
import org.unicore.resources.USpace;
import org.unicore.sets.ResourceEnumeration;
import org.unicore.sets.ResourceSet;
import com.pallas.unicore.requests.GetListing;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.threadpool.IObserver;

/**
 * Layer between RemoteFileSystemView and NJS on remote system.
 * 
 * @author Ralf Ratering
 * @version $Id: RemoteFileSystem.java,v 1.1 2004/05/25 14:58:55 rmenday Exp $
 */
public class RemoteFileSystem implements Serializable, IObserver {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.remotefilechooser");
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.remotefilechooser.ResourceStrings");
	/**
	 * define file separator on remote file system for now we assume that we
	 * only deal with remote UNIX systems
	 */
	public final static String SEPARATOR = "/";

	public static String getParentPath(String path) {
		int index = path.lastIndexOf(SEPARATOR);
		return path.substring(0, index);
	}
	private RemoteFile[] currentListing;
	private boolean getListingIsRunning = false;
	// file system home
	private RemoteFile home;
	// file system roots
	private RemoteFile[] roots;
	// available storage resources on vsite
	private Vector storages;
	// vsite to connect to
	private Vsite vsite;

	/**
	 * Constructor
	 * 
	 * @param vsite
	 *            to connect to
	 */
	public RemoteFileSystem(Vsite vsite) {
		this.vsite = vsite;
		updateStorageResources();
	}

	/**
	 * @return file system's home
	 */
	public RemoteFile getHome() {
		return home;
	}

	/**
	 * Submit a ListDirectory task to the NJS, wait for the result and return
	 * it.
	 * 
	 * @param dir
	 *            directory to be listed
	 * @return array with RemoteFiles from the directory
	 */
	public RemoteFile[] getListing(RemoteFile dir) {
		GetListing request = new GetListing(dir);
		getListingIsRunning = true;
		request.start();
		request.addObserver(this);
		// wait until thread returns and calls observableUpdate()
		while (getListingIsRunning) {
			try {
				logger.fine("Waiting for GetListing thread to return....");
				Thread.sleep(1000);
			}
			catch (InterruptedException e) {
				logger.log(Level.WARNING, "GetListing Thread was interrupted.",
						e);
			}
		}
		return currentListing;
	}

	public String getPath(Storage storage) {
		for (int i = 0; i < roots.length; i++) {
			if (roots[i].getStorage().equals(storage)) {
				return roots[i].getCompletePath();
			}
		}
		return null;
	}

	/**
	 * @return array containing file system roots
	 */
	public RemoteFile[] getRoots() {
		return roots;
	}

	public Vsite getVsite() {
		return vsite;
	}

	/**
	 * Check if a file is a file system root
	 * 
	 * @param f
	 *            RemoteFile to be tested
	 * @return true if f is a root
	 */
	public boolean isRoot(File f) {
		for (int i = 0; i < roots.length; i++) {
			if (f.equals(roots[i])) {
				return true;
			}
		}
		return false;
	}

	public void observableUpdate(Object observable, Object changeCode) {
		GetListing request = (GetListing) observable;
		currentListing = request.getRemoteFileArray();
		getListingIsRunning = false;
	}

	public void updatePath(RemoteFile file) {
		try {
			String parentSubspacePath = "";
			String subspacePath = file.getSubspacePath();
			int index = subspacePath.lastIndexOf(SEPARATOR);
			if (index > 0) {
				parentSubspacePath = subspacePath.substring(0, index);
			}
			RemoteFile parentDir = new RemoteFile(parentSubspacePath, file
					.getStorage(), file.getFileSystem());
			RemoteFile[] listing = getListing(parentDir);
			if (!subspacePath.equals("")) {
				for (int i = 0; i < listing.length; i++) {
					if (listing[i].getPath().endsWith(file.getName())) {
						file.setXFile(listing[i].getXFile());
						break;
					}
				}
			}
			// storage
			else {
				XFile xfile = listing[0].getXFile();
				String filePath = xfile.getName();
				index = filePath.lastIndexOf(SEPARATOR);
				String storagePath = filePath.substring(0, index - 1);
				file.getXFile().setName(storagePath);
			}
		}
		catch (Exception e) {
			logger.log(Level.WARNING, "Could not update path  for " + file, e);
		}
	}

	/**
	 * Update current storage resources at vsite and build new file system roots
	 * and home.
	 */
	private void updateStorageResources() {
		storages = new Vector();
		// build storage vector
		ResourceSet storageSet = ResourceManager.getResourceSet(vsite)
				.getStorageResources();
		ResourceEnumeration storageEnum = storageSet.elements();
		while (storageEnum.hasMoreElements()) {
			Storage storage = (Storage) storageEnum.nextElement();
			// do not handle uspace storage
			if (!(storage instanceof USpace
					|| storage instanceof AlternativeUspace || storage instanceof org.unicore.resources.Spool)) {
				storages.add(storage);
			}
		}
		if (storageSet == null) {
			roots = new RemoteFile[0];
			return;
		}
		// build roots array and find home
		roots = new RemoteFile[storages.size()];
		for (int i = 0; i < storages.size(); i++) {
			Storage s = (Storage) storages.elementAt(i);
			roots[i] = new RemoteFile(s, this);
			if (s instanceof Home) {
				home = roots[i];
			}
		}
		// no home available
		if (home == null) {
			home = roots[0];
		}
	}
}