/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.remotefilechooser;

import java.awt.Component;
import java.awt.Cursor;
import java.io.File;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import javax.swing.filechooser.FileSystemView;
import org.unicore.Vsite;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * FileSystemView on remote UNICORE file systems
 * 
 * @author Ralf Ratering
 * @version $Id: RemoteFileSystemView.java,v 1.5 2004/05/17 14:23:22 rraterin
 *          Exp $
 */
public class RemoteFileSystemView extends FileSystemView {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.remotefilechooser");
	// current directory
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.remotefilechooser.ResourceStrings");
	private RemoteFile currentDir;
	// file system that does the work
	private RemoteFileSystem fs;
	// flag if chooser has been initialized
	private boolean initialized = false;
	// parent component needed to switch mouse cursor
	private Component parent;

	/**
	 * Constructor
	 * 
	 * @param vsite
	 *            to connect to
	 * @param parent
	 *            GUI component
	 */
	public RemoteFileSystemView(Vsite vsite, Component parent) {
		this.parent = parent;
		this.fs = new RemoteFileSystem(vsite);
	}

	/**
	 * Build file object from directory and filename.
	 * 
	 * @param d
	 *            directory that contains file
	 * @param subspacePath
	 * @return new file object
	 */
	public File createFileObject(File d, String subspacePath) {
		RemoteFile dir = (RemoteFile) d;
		return new RemoteFile(subspacePath, dir.getStorage(), fs);
	}

	/**
	 * Not yet implemented
	 *  
	 */
	public File createNewFolder(File containingDir) throws java.io.IOException {
		logger.warning("Cannot create directories yet!");
		return null;
	}

	/**
	 * Gets the defaultDirectory = home
	 */
	public File getDefaultDirectory() {
		return getHomeDirectory();
	}

	/**
	 * Get array of files for a certain directory
	 * 
	 * @param dir
	 *            directory to get contents from
	 * @param useFileHiding
	 *            flag if we show hidden files or not
	 * @return array of files
	 */
	public File[] getFiles(File dir, boolean useFileHiding) {
		if (dir instanceof sun.awt.shell.ShellFolder) {
			return new File[0];
		}
		currentDir = (RemoteFile) dir;
		if (parent != null && initialized) {
			parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		}
		File[] listing = fs.getListing((RemoteFile) dir);
		if (parent != null) {
			parent.setCursor(Cursor.getDefaultCursor());
		}
		initialized = true;
		return listing;
	}

	/**
	 * Get file system home
	 */
	public File getHomeDirectory() {
		return fs.getHome();
	}

	/**
	 * Get parent directory from a file
	 * 
	 * @param d
	 *            RemoteFile to get parent from
	 * @return parent RemoteFile
	 */
	public File getParentDirectory(File d) {
		RemoteFile dir = (RemoteFile) d;
		return dir.getParentFile();
	}

	/**
	 * Get file system roots
	 * 
	 * @return array containing root RemoteFiles
	 */
	public File[] getRoots() {
		return fs.getRoots();
	}

	/**
	 * Get System name for a File
	 * 
	 * @param file
	 *            input file
	 * 
	 * @return The systemDisplayName value
	 */
	public String getSystemDisplayName(File file) {
		String name = "";
		try {
			name = file.getName();
		}
		catch (Exception e) {
			logger.log(Level.WARNING,
					"Could not get system display name for file " + file, e);
		}
		return name;
	}

	/**
	 * Get systemIcon for a file
	 */
	public Icon getSystemIcon(File file) {
		if (file.isDirectory()) {
			return ResourceManager.getIcon(ResourceManager.OPEN);
		}
		else {
			return ResourceManager.getIcon(ResourceManager.NEW);
		}
	}

	/**
	 * Gets the system type description for a File
	 */
	public String getSystemTypeDescription(File file) {
		if (file.isDirectory()) {
			return "Directory";
		}
		else {
			return "File";
		}
	}

	/**
	 * Is File a hidden one?
	 */
	public boolean isHiddenFile(File f) {
		return f.isHidden();
	}

	/**
	 * Is File a Root?
	 */
	public boolean isRoot(File f) {
		if (f instanceof sun.awt.shell.ShellFolder) {
			return false;
		}
		return fs.isRoot(f);
	}

	/**
	 * Is File traversable?
	 */
	public Boolean isTraversable(File file) {
		RemoteFile remoteFile = (RemoteFile) file;
		return Boolean.valueOf(remoteFile.isDirectory() || remoteFile.isRoot());
	}
}