/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.tables;

import java.awt.Component;
import java.util.HashSet;
import java.util.Iterator;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

/**
 * @author Klaus-Dieter Oertel
 * @version $Id: AdjustTable.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class AdjustTable extends JTable {

	private HashSet ignoreDataColumnSet = new HashSet();
	private HashSet ignoreHeaderColumnSet = new HashSet();

	public AdjustTable() {
		super();
	}

	public AdjustTable(TableModel tableModel) {
		super(tableModel);
	}

	public void adjustWidths() {
		adjustWidths(getAdjustWidths());
	}

	public void adjustWidths(int[] widths) {
		boolean adjustHeaderCols[] = getAdjustColumns(ignoreHeaderColumnSet);
		boolean adjustDataCols[] = getAdjustColumns(ignoreDataColumnSet);
		for (int j = 0; j < adjustHeaderCols.length; j++) {
			if (adjustHeaderCols[j] || adjustDataCols[j]) {
				this.getColumnModel().getColumn(j).setPreferredWidth(widths[j]);
			}
		}
	}

	public void debugWidths(String type, int widths[]) {
		System.err.println(type + " WIDTH");
		for (int j = 0; j < widths.length; j++) {
			System.err.println(" width[" + j + "]=" + widths[j]);
		}
		System.err.println("FOUND");
	}

	private boolean[] getAdjustColumns(HashSet ignoreColumns) {
		boolean adjustColumns[] = new boolean[getTableHeader().getColumnModel()
				.getColumnCount()];
		for (int j = 0; j < adjustColumns.length; j++) {
			adjustColumns[j] = true;
		}
		Iterator iterator = ignoreColumns.iterator();
		while (iterator.hasNext()) {
			int j = ((Integer) iterator.next()).intValue();
			adjustColumns[j] = false;
		}
		return adjustColumns;
	}

	public int[] getAdjustWidths() {
		int widths[] = getHeaderWidths();
		int dataWidths[] = getDataWidths();
		for (int j = 0; j < widths.length; j++) {
			widths[j] = Math.max(widths[j], dataWidths[j]);
		}
		return widths;
	}

	public int[] getDataWidths() {
		TableModel model = this.getModel();
		int nRows = model.getRowCount();
		int nCols = model.getColumnCount();
		int widths[] = new int[nCols];
		for (int j = 0; j < nCols; j++) {
			widths[j] = -1;
		}
		if (nRows == 0) {
			return widths;
		}
		TableCellRenderer renderer[] = new TableCellRenderer[nCols];
		for (int j = 0; j < nCols; j++) {
			renderer[j] = this.getCellRenderer(0, j);
		}
		boolean adjustColumns[] = getAdjustColumns(ignoreDataColumnSet);
		for (int i = 0; i < nRows; i++) {
			for (int j = 0; j < nCols; j++) {
				if (adjustColumns[j]) {
					Object data = model.getValueAt(i, j);
					if (data != null) {
						Component cell = renderer[j]
								.getTableCellRendererComponent(this, data,
										false, false, i, j);
						int width = cell.getPreferredSize().width;
						widths[j] = Math.max(widths[j], width);
					}
				}
			}
		}
		return widths;
	}

	public int[] getHeaderWidths() {
		JTableHeader tableHeader = this.getTableHeader();
		TableColumnModel headerColumnModel = tableHeader.getColumnModel();
		int nCols = headerColumnModel.getColumnCount();
		int widths[] = new int[nCols];
		for (int j = 0; j < nCols; j++) {
			widths[j] = -1;
		}
		boolean adjustColumns[] = getAdjustColumns(ignoreHeaderColumnSet);
		TableCellRenderer defaultHeaderRenderer = tableHeader
				.getDefaultRenderer();
		for (int j = 0; j < nCols; j++) {
			if (adjustColumns[j]) {
				TableCellRenderer headerRenderer = headerColumnModel.getColumn(
						j).getHeaderRenderer();
				if (headerRenderer == null) {
					headerRenderer = defaultHeaderRenderer;
				}
				Object data = this.getColumnName(j);
				if (data != null) {
					Component cell = headerRenderer
							.getTableCellRendererComponent(this, data, false,
									false, 0, j);
					widths[j] = cell.getPreferredSize().width;
				}
			}
		}
		return widths;
	}

	public void ignoreDataWidth(int column) {
		ignoreDataColumnSet.add(new Integer(column));
	}

	public void ignoreHeaderWidth(int column) {
		ignoreHeaderColumnSet.add(new Integer(column));
	}

	public void useDataWidth(int column) {
		ignoreDataColumnSet.remove(new Integer(column));
	}

	public void useHeaderWidth(int column) {
		ignoreHeaderColumnSet.remove(new Integer(column));
	}
}