/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.trees;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.JobContainer;

/**
 * Node that extends a panel node by storing an ActionContainer. Note, that the
 * state of the node follows the container's state and also the node icon is
 * taken from the container. This class implements the Clonable interface to
 * allow copying to the clipboard.
 * 
 * @author Ralf Ratering
 * @version $Id: ContainerNode.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class ContainerNode extends PanelNode {

	private int completedDownloads;
	private int totalDownloads;

	/**
	 * Constructor
	 * 
	 * @param container
	 *            to be stored
	 * @param panel
	 *            belonging to this node
	 */
	public ContainerNode(ActionContainer container, JPanel panel) {
		super(container, panel);
	}

	/**
	 * Get a reference to the ActionContainer
	 * 
	 * @return stored ActionContainer
	 */
	public ActionContainer getActionContainer() {
		return (ActionContainer) userObject;
	}

	public int getCompletedDownloads() {
		return this.completedDownloads;
	}

	/**
	 * Overwrite method to pipe through icon from container
	 * 
	 * @return The ImageIcon value
	 */
	public ImageIcon getCurrentIcon() {
		ActionContainer container = (ActionContainer) userObject;
		return container.getStatusIcon();
	}

	/**
	 * Get the state of a JobContainer. All other containers are always up to
	 * date.
	 * 
	 * @return one of the constants ActionContainer.STATE_UPDATING, etc.
	 */
	public int getState() {
		if (getActionContainer() instanceof JobContainer) {
			return ((JobContainer) getActionContainer()).getState();
		}
		else {
			return ActionContainer.STATE_UP_TO_DATE;
		}
	}

	public int getTotalDownloads() {
		return this.totalDownloads;
	}

	/**
	 * Test if a container is currently updating
	 * 
	 * @return true, if container is updating
	 */
	public boolean isUpdating() {
		return (getState() == ActionContainer.STATE_UPDATING);
	}

	/**
	 * Test if a container is up to date
	 * 
	 * @return true if container is up to date
	 */
	public boolean isUpToDate() {
		return (getState() == ActionContainer.STATE_UP_TO_DATE);
	}

	public synchronized void setCompletedDownloads(int completedDownloads) {
		this.completedDownloads = completedDownloads;
	}

	public synchronized void incrementCompletedDownloads() {
		this.completedDownloads++;
	}

	/**
	 * Set state for a job container. In case node does not represent a job
	 * group do nothing.
	 * 
	 * @param newState
	 *            The new state value
	 */
	public synchronized void setState(int newState) {
		if (userObject instanceof JobContainer) {
			((JobContainer) userObject).setState(newState);
		}
		super.setState(newState);
	}

	public void setTotalDownloads(int totalDownloads) {
		this.totalDownloads = totalDownloads;
	}

	/**
	 * Set container state to STATE_UP_TO_DATE
	 */
	public void setUpToDate() {
		setState(ActionContainer.STATE_UP_TO_DATE);
	}
}