/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.client.xml;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.CRC32;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import com.pallas.unicore.client.Client;
import com.pallas.unicore.client.util.PluginManager;
import com.pallas.unicore.client.util.UnicorePlugin;
import com.pallas.unicore.utility.UserMessages;

/**
 * @author Klaus-Dieter Oertel
 * @version $Id: XMLObjectIO.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class XMLObjectIO {

	class DebugMessages {

		private StringBuffer indent;
		private final String INDENT_BASE;
		private final String INDENT_INCR;
		private final int INDENT_INCR_LENGTH;

		DebugMessages(String base, String incr) {
			INDENT_BASE = base;
			INDENT_INCR = incr;
			INDENT_INCR_LENGTH = INDENT_INCR.length();
			indent = new StringBuffer(INDENT_BASE);
		}

		public void popIndent() {
			indent.setLength(indent.length() - INDENT_INCR_LENGTH);
		}

		public void println(int level, String message) {
			if (DEBUG >= level) {
				if (indent.length() > 0) {
					System.err.println(indent + " " + message);
				}
				else {
					System.err.println(indent + message);
				}
			}
		}

		public void println(String message) {
			println(1, message);
		}

		public void pushIndent() {
			indent.append(INDENT_INCR);
		}
	}
	public final static Integer DEBUGXML = Integer.getInteger("DEBUGXML");
	public final static int DEBUG = (DEBUGXML == null) ? 0 : DEBUGXML
			.intValue();
	final static boolean FIELDTYPE;
	static {
		if (DEBUG > 2) {
			FIELDTYPE = true;
		}
		else {
			FIELDTYPE = false;
		}
	}
	final static String ATTR_CHECKSUM = "CHECKSUM";
	final static String ATTR_DIM = "dim";
	final static String ATTR_ELEMVERSION = "XMLV";
	final static String ATTR_FIELDTYPE = "fieldtype";
	final static String ATTR_HASHKEY = "key";
	final static String ATTR_HASHVALUE = "value";
	final static String ATTR_ID = "ID";
	final static String ATTR_IDREF = "IDREF";
	final static String ATTR_INDEX = "i";
	final static String ATTR_LENGTH = "length";
	final static String ATTR_SIZE = "size";
	final static String ATTR_VALUETYPE = "valuetype";
	final static String ATTR_VERSION = "XMLVERSION";
	protected static Logger logger = Logger
			.getLogger("com.pallas.unicore.client.xml");
	// Static and transient fields are not serialized in XML format
	final static int NO_XML = Modifier.STATIC | Modifier.TRANSIENT;
	final static String NOTYETIMPL_INDICATOR = "-NOT-YET-IMPLEMENTED";
	final static String NULL_INDICATOR = "-NULL";
	static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.client.xml.ResourceStrings");
	final static String TAG_BINARY = "BINARY-";
	final static String TAG_UNICORE = "UNICOREpro";
	// TIMING level
	final static int TIMING = (Integer.getInteger("TIMING") == null)
			? 0
			: Integer.getInteger("TIMING").intValue();
	// Determine type of XML text entry to generate valid XML
	final static boolean USE_CDATA_SECTION = false;

	/*
	 * ASSUMPTION: Called within priviledged access block. Needed to set
	 * declared methods and to set the accessibility bit.
	 */
	protected final static Method getDeclaredMethod(Class ofClass,
			String methodName, Class[] args) {
		Method method = null;
		try {
			method = ofClass.getDeclaredMethod(methodName, args);
			if (method != null) {
				//		method.setAccessible(true);
			}
			else {
				throw new NoSuchMethodException("METHOD " + methodName
						+ " NOT FOUND");
			}
		}
		catch (NoSuchMethodException e) {
			//	    UserMessages.error(e.toString());
			if (DEBUG >= 2) {
				System.err.println("LOOKUP RESULT for " + ofClass.getName()
						+ ": no method " + methodName + " found");
			}
			//	    logger.log(Level.SEVERE, "", e);
			// Since it is alright if methodName does not exist,
			// no need to do anything special here.
		}
		return method;
	}

	/*
	 * ASSUMPTION: Called within priviledged access block. Needed to set
	 * declared methods and to set the accessibility bit.
	 */
	protected final static Method getDeclaredMethod(Class ofClass,
			String methodName, Class[] args, int requiredModifierMask,
			int disallowedModifierMask) {
		Method method = null;
		try {
			method = ofClass.getDeclaredMethod(methodName, args);
			if (method != null) {
				int mods = method.getModifiers();
				if ((mods & disallowedModifierMask) != 0
						|| (mods & requiredModifierMask) != requiredModifierMask) {
					if (DEBUG >= 2) {
						UserMessages.warning("Found method " + method
								+ " with wrong modifiers");
					}
					method = null;
				}
				else {
					//		    method.setAccessible(true);
				}
			}
		}
		catch (NoSuchMethodException e) {
			//	    UserMessages.error(e.toString());
			if (DEBUG >= 2) {
				System.err.println("LOOKUP RESULT for " + ofClass.getName()
						+ ": no method " + methodName + " found");
			}
			//	    logger.log(Level.SEVERE, "", e);
			// Since it is alright if methodName does not exist,
			// no need to do anything special here.
		}
		return method;
	}
	public DebugMessages debug = new DebugMessages("", "#");
	protected StringBuffer indent = new StringBuffer("|");
	protected TransformerFactory tFactory;

	protected XMLObjectIO() throws TransformerConfigurationException {
		tFactory = TransformerFactory.newInstance();
		if (!tFactory.getFeature(DOMSource.FEATURE)) {
			UserMessages
					.error("Internal library problem: saving in XML format impossible");
			throw new TransformerConfigurationException("No DOMSource");
		}
		if (!tFactory.getFeature(DOMResult.FEATURE)) {
			UserMessages
					.error("Internal library problem: loading from XML format impossible");
			throw new TransformerConfigurationException("No DOMResult");
		}
		if (!tFactory.getFeature(StreamSource.FEATURE)) {
			UserMessages
					.error("Internal library problem: loading from XML format impossible");
			throw new TransformerConfigurationException("No StreamSource");
		}
		if (!tFactory.getFeature(StreamResult.FEATURE)) {
			UserMessages
					.error("Internal library problem: saving in XML format failed");
			throw new TransformerConfigurationException("No StreamResult");
		}
	}

	protected final int checkDim(Class ctype) throws IOException {
		String type = null;
		int depth = 0;
		Class atype = ctype;
		while (atype.isArray()) {
			depth++;
			atype = atype.getComponentType();
		}
		//      if (depth > 1)
		//          throw new IOException( "more than 1 array dimension found [" + depth
		// + "]");
		return depth;
	}

	protected final void dumpDOM(Element elem) {
		System.out.println(indent + elem.getNodeName() + " % "
				+ elem.getLocalName());
		dumpDOM((Node) elem);
	}

	protected final void dumpDOM(Node node) {
		NodeList list = node.getChildNodes();
		indent.append(">>");
		for (int i = 0; i < list.getLength(); i++) {
			Node listi = list.item(i);
			if (listi.getNodeType() == Node.ELEMENT_NODE) {
				dumpDOM((Element) listi);
			}
			else if (listi.getNodeType() == Node.COMMENT_NODE) {
				//System.out.println(indent+"------------- COMMENT_NODE:
				// "+listi.getNodeValue());
			}
			else if (listi.getNodeType() == Node.TEXT_NODE) {
				String value = listi.getNodeValue();
				if (!value.startsWith("\n") || value.trim().length() > 0) {
					System.out.println(indent + "------------- TEXT_NODE: "
							+ value);
				}
			}
			else {
				System.out.println(indent + "------------- UNKNOWN: "
						+ listi.getNodeType());
				UserMessages.error("UNKNOWN: " + listi.getNodeType());
			}
		}
		indent.setLength(indent.length() - 2);
	}

	// Map valid XML format to string
	protected final String fromvalidXML(String in) {
		if (true) {
			return in;
		}
		String out = new String();
		int i = 0;
		while (i < in.length()) {
			char c = in.charAt(i);
			i++;
			if (c == '&') {
				if (in.startsWith("amp;", i)) {
					out += '&';
					i += 4;
				}
				else if (in.startsWith("lt;", i)) {
					out += '<';
					i += 3;
				}
				else if (in.startsWith("gt;", i)) {
					out += '>';
					i += 3;
				}
				else if (in.startsWith("apos;", i)) {
					out += '\'';
					i += 5;
				}
				else if (in.startsWith("quot;", i)) {
					out += '"';
					i += 5;
				}
			}
			else {
				out += c;
			}
		}
		return out;
	}

	// calculate a check sum from a DOM tree
	protected long getCheckSum(Node node) {
		long timing = System.currentTimeMillis();
		CRC32 chksum = new CRC32();
		try {
			Transformer transformer = tFactory.newTransformer();
			DOMSource checkSource = new DOMSource(node);
			ByteArrayOutputStream checkBytes = new ByteArrayOutputStream();
			StreamResult checkResult = new StreamResult(checkBytes);
			transformer.transform(checkSource, checkResult);
			checkBytes.close();
			if (DEBUG >= 3) {
				checkBytes.writeTo(new DataOutputStream(new FileOutputStream(
						"tttreadBytes.xml")));
			}
			chksum.update(checkBytes.toByteArray());
		}
		catch (TransformerConfigurationException tce) {
			// Error generated by the parser
			logger.severe("\n** Transformer Factory error");
			logger.severe("   " + tce.getMessage());
			// Use the contained exception, if any
			Throwable x = tce;
			if (tce.getException() != null) {
				x = tce.getException();
			}
			logger.log(Level.SEVERE, "", x);
		}
		catch (TransformerException te) {
			// Error generated by the parser
			logger.severe("\n** Transformation error");
			logger.severe("   " + te.getMessage());
			// Use the contained exception, if any
			Throwable x = te;
			if (te.getException() != null) {
				x = te.getException();
			}
			logger.log(Level.SEVERE, "", x);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "", e);
		}
		timing = System.currentTimeMillis() - timing;
		if (TIMING > 2) {
			logger.info("TIMING[3]: checksum calculated in " + timing
					+ " milliseconds");
		}
		return chksum.getValue();
	}

	protected final String getTypeName(Class ctype) throws IOException {
		String type = null;
		if (ctype.isPrimitive()) {
			type = ctype.getName();
		}
		else if (ctype.isArray()) {
			int depth = 0;
			Class atype = ctype;
			while (atype.isArray()) {
				depth++;
				atype = atype.getComponentType();
				type = getTypeName(atype);
			}
			if (depth > 1) {
				throw new IOException("more than 1 array dimension found ["
						+ depth + "]");
			}
		}
		else {
			type = (ctype.isInstance(new String()))
					? "xString:" + ctype
					: ctype.getName();
		}
		return type;
	}

	public final Class resolveClass(String className) throws IOException,
			ClassNotFoundException {
		// system classloader
		Class self = XMLObjectIO.class;
		ClassLoader loader = self.getClassLoader();
		PluginManager pManager = Client.getPluginManager();
		if (pManager == null) {
			// system class loader
			return Class.forName(className, false, loader);
		}
		Vector knownPlugins = pManager.getPlugins();
		if (!knownPlugins.isEmpty()) {
			/*
			 * iterate through the plugins and try to load classes with the
			 * associated classloader
			 */
			for (int i = 0; i < knownPlugins.size(); i++) {
				UnicorePlugin plugin = (UnicorePlugin) knownPlugins
						.elementAt(i);
				loader = plugin.getPluginClassloader();
				try {
					Class retClass = Class.forName(className, false, loader);
					//				  System.out.println("Loading class: "+retClass);
					return retClass;
				}
				catch (ClassNotFoundException cnfe) {
				}
			}
		}
		else {
			// system class loader
			return Class.forName(className, false, loader);
		}
		// finally class not found
		throw new ClassNotFoundException(className + res.getString("NOT_FOUND"));
	}

	protected final String validClassName(String xmlTag) {
		String className;
		if (xmlTag.indexOf(".") >= 0 || xmlTag.indexOf("_") >= 0) {
			debug.println(2, "Generating class name from valid XML tag "
					+ xmlTag);
			// Handle names of internal and array classes
			int pos = xmlTag.lastIndexOf('.');
			StringBuffer buf = new StringBuffer(xmlTag.substring(0, pos));
			buf.append('$');
			buf.append(xmlTag.substring(pos + 1));
			className = buf.toString();
			className = className.replace('_', '[').replace(' ', ';');
			debug.println(2, "Generating class object from valid class name "
					+ className);
		}
		else {
			className = new String(xmlTag);
		}
		return className;
	}

	// Map string to valid XML format
	protected final String validXML(String in) {
		if (true) {
			return in;
		}
		String out = new String("");
		for (int i = 0; i < in.length(); i++) {
			switch (in.charAt(i)) {
				case '&' :
					out += "&amp;";
					break;
				case '<' :
					out += "&lt;";
					break;
				case '>' :
					out += "&gt;";
					break;
				case '\'' :
					out += "&apos;";
					break;
				case '"' :
					out += "&quot;";
					break;
				default :
					out += in.charAt(i);
			}
		}
		return out;
	}

	protected final String validXMLTag(Class clazz) {
		String className = clazz.getName();
		if (className.indexOf("$") >= 0 || className.indexOf("[") >= 0) {
			debug.println(2, "Generating valid XML tag from  " + className);
			// Handle names of internal and array classes
			className = className.replace('$', '.').replace('[', '_').replace(
					';', ' ').trim();
			debug.println(2, "Generating final valid XML tag " + className);
		}
		return className;
	}

	protected final String validXMLTag(Object obj) {
		return validXMLTag(obj.getClass());
	}
}