/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.container;

import java.util.ResourceBundle;
import java.util.Vector;
import org.unicore.ajo.ActionGroup;
import org.unicore.resources.Resource;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.extensions.FileTransfer;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.TransferFactory;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Container for all tasks that transfer files from one uspace to another
 * 
 * @author Ralf Ratering
 * @version $Id: FileTransferContainer.java,v 1.1 2004/05/25 14:58:50 rmenday Exp $
 */
public class FileTransferContainer extends FileContainer {

	private static ResourceBundle res = ResourceBundle
			.getBundle("com.pallas.unicore.container.ResourceStrings");
	final static long serialVersionUID = -7293842589398013001L;
	private Vector additionalDependencies;

	/**
	 * Constructor builds file container as child of job container
	 * 
	 * @param parentContainer
	 *            parent job group
	 */
	public FileTransferContainer(GroupContainer parentContainer) {
		super(parentContainer);
	}

	/**
	 * Get Action group containing all tasks from this container
	 */
	public void buildActionGroup() {
		actionGroup = new ActionGroup(getName());
		// import files
		TransferFactory transferFactory = new TransferFactory(
				getFileTransfers(), actionGroup, this);
		transferFactory.build();
		// copy identifier from action group to container to to have a unique id
		// for the njs.
		syncIdentifiers();
	}

	/**
	 * Add container specific errors to those of super classes
	 * 
	 * @return ErrorSet
	 */
	public ErrorSet checkContents() {
		ErrorSet errorSet = super.checkContents();
		// remove previous additional dep
		if (additionalDependencies != null) {
			for (int i = 0; i < additionalDependencies.size(); i++) {
				getParentContainer().removeDependency(
						(DependencyContainer) additionalDependencies
								.elementAt(i));
			}
		}
		additionalDependencies = new Vector();
		if (fileTransfers == null || fileTransfers.length == 0) {
			errorSet.add(new UError(getIdentifier(), res
					.getString("NO_TRANSFERS")));
		}
		// check if destination jobs are existing
		else {
			NamedResourceSet resourceSet = ResourceManager.getResourceSet(this
					.getVsite());
			for (int i = 0; i < fileTransfers.length; i++) {
				FileTransfer fileTransfer = fileTransfers[i];
				String message = res.getString("INCOMPLETE_FILE_TRANSFER")
						+ fileTransfer + res.getString("AT_ROW") + (i + 1)
						+ ". ";
				if (!fileTransfer.isComplete()) {
					errorSet.add(new UError(id, message));
				}
				Resource sourceStorage = (Resource) fileTransfer
						.getStorageItem();
				if (sourceStorage == null
						|| !resourceSet.contains(sourceStorage)) {
					errorSet
							.add(new UError(id, message + "\n\t"
									+ res.getString("STORAGE")
									+ fileTransfer.getStorageItem()
									+ res.getString("NOT_AVAILABLE")
									+ this.getVsite()));
				}
				JobContainer destJob = fileTransfer.getDestinationJob();
				if (destJob == null || !jobExists(destJob)) {
					errorSet.add(new UError(getIdentifier(), message + "\n\t"
							+ res.getString("DESTJOB_MISSING") + destJob));
					// skip remaining tests which require a valid destJob
					continue;
				}
				if (destJob == this.getParentJobContainer()) {
					errorSet.add(new UError(getIdentifier(), message + "\n\t"
							+ res.getString("DESTJOB_PARENT") + destJob));
				}
				NamedResourceSet destResourceSet = ResourceManager
						.getResourceSet(destJob.getVsite());
				Resource targetStorage = (Resource) fileTransfer
						.getOtherStorage();
				if (targetStorage == null
						|| !destResourceSet.contains(targetStorage)) {
					errorSet
							.add(new UError(id, message + "\n\t"
									+ res.getString("STORAGE") + targetStorage
									+ res.getString("NOT_AVAILABLE")
									+ this.getVsite()));
				}
				// we'll do this later on...
				/*
				 * else if(!isPredecessorOf(destJob)) { DependencyContainer dep =
				 * new DependencyContainer(this, destJob);
				 * additionalDependencies.add(dep);
				 * getParentContainer().addDependency(dep); }
				 */
			}
		}
		setErrors(errorSet);
		return errorSet;
	}

	/**
	 * Gets the jobExisting attribute of the FileTransferContainer object
	 * 
	 * @param job
	 *            Description of the Parameter
	 * @return The jobExisting value
	 */
	private boolean jobExists(JobContainer job) {
		return findTask(findRootContainer(this), job);
	}
}