/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.extensions;

import java.util.Vector;

import org.unicore.Vsite;
import org.unicore.ajo.FileCheck;
import org.unicore.resources.Storage;
import org.unicore.sets.ResourceSet;

import com.pallas.unicore.container.ActionContainer;
import com.pallas.unicore.container.errorspec.ErrorSet;
import com.pallas.unicore.container.errorspec.UError;
import com.pallas.unicore.resourcemanager.ResourceManager;

/**
 * Test files for existence and properties
 * 
 * @author Ralf Ratering
 * @version $Id: FileTest.java,v 1.1 2004/05/25 14:58:45 rmenday Exp $
 */
public class FileTest extends AbstractTest {

	/**
	 *  File exists?
	 */
	public final static FileTestType EXISTING =
		new FileTestType(FileTestType.EXISTING);

	/**
	 *  File doesn't exist?
	 */
	public final static FileTestType NOT_EXISTING =
		new FileTestType(FileTestType.NOT_EXISTING);

	/**
	 *  File readable?
	 */
	public final static FileTestType READABLE =
		new FileTestType(FileTestType.READABLE);

	/**
	 *  File writable?
	 */
	public final static FileTestType WRITABLE =
		new FileTestType(FileTestType.WRITABLE);

	/**
	 *  File executable?
	 */
	public final static FileTestType EXECUTABLE =
		new FileTestType(FileTestType.EXECUTABLE);

	final static long serialVersionUID = 8558445171865314876L;

	private static Vector types;
	private Storage storage;
	private String subspacePath;

	/**
	 *  Get all types
	 *
	 *@return    Vector containing types
	 */
	public static Vector getTypes() {
		if (types == null) {
			types = new Vector();
			types.add(DISABLED);
			types.add(EXISTING);
			types.add(NOT_EXISTING);
			types.add(READABLE);
			types.add(WRITABLE);
			types.add(EXECUTABLE);
		}
		return types;
	}

	
	/**
	 *  check test parameters
	 *
	 *@param  errorSet   ErrorSet that errors will be added to
	 *@param  container  container belonging to test
	 *@return            updated error set
	 */
	public ErrorSet checkContents(
		ErrorSet errorSet,
		ActionContainer container) {
		super.checkContents(errorSet, container);

		Vsite vsite = container.getParentJobContainer().getVsite();
		if (vsite == null) {
			return errorSet;
		}

		NamedResourceSet siteResources = ResourceManager.getResourceSet(vsite);
		if (storage == null) {
			errorSet.add(
				new UError(
					container.getIdentifier(),
					"No storage location selected."));
		}
		else if (!siteResources.contains(storage)) {
			errorSet.add(
				new UError(
					container.getIdentifier(),
					"Storage location "
						+ storage
						+ " not available at site "
						+ vsite
						+ "."));
		}

		return errorSet;
	}

	/**
	 *  Build an AJO FileCheck task from test parameters
	 *
	 *@return    an AJO FileCheck task
	 */
	public FileCheck getFileCheckTask() {
		FileCheck fileCheck =
			new FileCheck(
				"FileCheck_" + ResourceManager.getNextObjectIdentifier());
		fileCheck.setFileToCheck(getSubspacePath());
		fileCheck.setResources(new ResourceSet(getStorage()));
		if (type.equals(EXISTING)) {
			fileCheck.setFileExists(Boolean.TRUE);
		}
		else if (type.equals(NOT_EXISTING)) {
			fileCheck.setFileExists(Boolean.FALSE);
		}
		else if (type.equals(READABLE)) {
			fileCheck.setCanReadFile(Boolean.TRUE);
		}
		else if (type.equals(WRITABLE)) {
			fileCheck.setCanWriteFile(Boolean.TRUE);
		}
		else if (type.equals(EXECUTABLE)) {
			fileCheck.setCanExecuteFile(Boolean.TRUE);
		}
		return fileCheck;
	}

	/**
	 *  Get storage resource for file
	 *
	 *@return    Storage resource
	 */
	public Storage getStorage() {
		return storage;
	}

	/**
	 *  Get file path within storage resource
	 *
	 *@return    subspace path
	 */
	public String getSubspacePath() {
		return subspacePath;
	}

	/**
	 *  Set file storage resource
	 *
	 *@param  storage  File Storage Resource
	 */
	public void setStorage(Storage storage) {
		this.storage = storage;
	}

	/**
	 *  Set file path within storage resource
	 *
	 *@param  subspacePath  subspace path
	 */
	public void setSubspacePath(String subspacePath) {
		this.subspacePath = subspacePath;
	}

	/**
	 *  Overwrite super class method
	 *
	 *@return    String description
	 */
	public String toString() {
		return "Test if file "
			+ subspacePath
			+ " at "
			+ storage
			+ " is "
			+ type;
	}

}
