/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.ObjectInputStream;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyActionOutcome;
import org.unicore.outcome.AbstractActionStatus;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcome;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.connection.Connection;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.extensions.FileStorage;
import com.pallas.unicore.resourcemanager.OutcomeManager;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.security.JobConverter;
import com.pallas.unicore.utility.DeleteAll;
import com.pallas.unicore.utility.UserMessages;

/**
 * Get the outcome for a node in the job monitor
 * 
 * @author Thomas Kentemich
 * @version $Id: GetOutcome.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetOutcome extends ObservableNodeThread {

	private String nspaceString = FileStorage.NSPACE_STRING;

	/**
	 * Constructor
	 * 
	 * @param node
	 *            container node in job monitor
	 */
	public GetOutcome(ContainerNode node) {
		super(node);
	}

	private AbstractJob assembleRecursiveJobOutcome(AbstractJob root,
			JobContainer jc) {
		// must construct an abstract job wrapper; consign it in non-polling
		// mode
		Vsite vsite = jc.getVsite();
		AbstractJob job = new AbstractJob(servicePrefix + "_JobStatus_"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				ResourceManager.getUser(vsite));
		CopyActionOutcome cao = new CopyActionOutcome("CopyActionOutcome", root
				.getId());
		job.add(cao);
		// append the UPL directory of the CopyActionOutcome in order to find
		// the streamed files
		jc.setLastUpdateUPLDirectory(cao.getUPLDirectoryName());
		return job;
	}

	/**
	 * Start the Thread
	 */
	public void run() {
		// the job outcome
		AbstractJob_Outcome aj_out = null;
		try {
			//generating the job
			//first, the " GetJobs" task
			JobContainer jobContainer = (JobContainer) getNode()
					.getUserObject();
			PanelNode vsiteNode = (PanelNode) getNode().getParent();
			Vsite vsite = null;
			// if we have a vsite node it's safest to use the underlying vsite
			// object
			if (vsiteNode != null) {
				vsite = (Vsite) vsiteNode.getUserObject();
			}
			// service jobs won't have vsite nodes, so take the container's
			// vsite
			else {
				vsite = jobContainer.getVsite();
			}
			RetrieveOutcome ro = new RetrieveOutcome(
					(AJOIdentifier) jobContainer.getIdentifier(), vsite);
			JobConverter.convert(ro, true);
			Connection connection = ResourceManager.getConnectionManager()
					.lookupConnection(vsite, ResourceManager.getUser(vsite));
			Reply troReply = connection.sendMessage(ro, null);
			int returnCode = troReply.getLastEntry().getReturnCode();
			// now analyze outcome
			if (troReply == null || !(troReply instanceof RetrieveOutcomeReply)) {
				throw new Exception("Unexpected type of outcome.");
			}
			else {
				if (jobContainer.getLastUpdateUPLDirectory() != null) {
					DeleteAll.deleteAll(new File(Connection.getSessionDir()
							+ File.separator
							+ jobContainer.getLastUpdateUPLDirectory()));
				}
				RetrieveOutcomeReply roReply = (RetrieveOutcomeReply) connection
						.getReply();
				// deserialize the Outcome Object
				Outcome outcome = (Outcome) new ObjectInputStream(
						new ByteArrayInputStream(roReply.getOutcome()))
						.readObject();
				if (outcome instanceof AbstractJob_Outcome) {
					aj_out = ConsignForm
							.convertFrom((AbstractJob_Outcome) outcome);
				}
				else {
					throw new Exception("Unexpected type of outcome.");
				}
			}
			// if job is still executing send a CopyActionOutcome request to
			// retrieve stdout/stderr as streamed files
			if (jobContainer.getStatus().isEquivalent(
					AbstractActionStatus.EXECUTING)) {
				AbstractJob job = assembleRecursiveJobOutcome(
						(AbstractJob) jobContainer.getActionGroup(),
						jobContainer);
				troReply = nonPolling(job, vsite, ResourceManager
						.getUser(vsite));
			}
			// put outcome into outcome manager after streamed files have
			// arrived!
			OutcomeManager.setOutcome(aj_out, jobContainer);
		}
		catch (Exception e) {
			UserMessages.warning("Getting outcome for job " + getNode()
					+ " failed.");
			logger.log(Level.SEVERE, "Getting outcome for job " + getNode()
					+ " failed.", e);
		}
		// notify all my observers on task end
		notifyObservers(this, aj_out);
	}
}