/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.requests;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.Vector;
import java.util.logging.Level;
import org.unicore.AJOIdentifier;
import org.unicore.User;
import org.unicore.Vsite;
import org.unicore.ajo.AbstractJob;
import org.unicore.ajo.CopyJob;
import org.unicore.ajo.GetJobs;
import org.unicore.outcome.AbstractJob_Outcome;
import org.unicore.outcome.CopyJob_Outcome;
import org.unicore.outcome.GetJobs_Outcome;
import org.unicore.outcome.Outcome;
import org.unicore.sets.AJOIdentifierEnumeration;
import org.unicore.sets.AJOIdentifierSet;
import org.unicore.sets.OutcomeEnumeration;
import org.unicore.upl.Reply;
import org.unicore.upl.RetrieveOutcomeReply;
import org.unicore.utility.ConsignForm;
import com.pallas.unicore.client.trees.ContainerNode;
import com.pallas.unicore.client.trees.PanelNode;
import com.pallas.unicore.container.AjoContainer;
import com.pallas.unicore.container.JobContainer;
import com.pallas.unicore.resourcemanager.ResourceManager;
import com.pallas.unicore.utility.UserMessages;
import com.pallas.unicore.utility.VectorSorter;

/**
 * Get a Vector containing all jobs at a specific virtual site
 * 
 * @author Thomas Kentemich
 * @author Ralf Ratering
 * @version $Id: GetRunningJobs.java,v 1.1 2004/05/25 14:58:54 rmenday Exp $
 */
public class GetRunningJobs extends ObservableNodeThread {

	private Vector knownJobs;

	/**
	 * Constructor
	 * 
	 * @param node
	 *            Vsite node in Job Monitor to get jobs from
	 */
	public GetRunningJobs(PanelNode node) {
		super(node);
	}

	private Vector getAbstractJobs(AJOIdentifierSet jobIds) throws Exception {
		// build return vector
		Vector abstractJobs = new Vector();
		if (jobIds == null) {
			return abstractJobs;
		}
		// build a job to get AbstractJob objects belonging to ids
		Vsite vsite = (Vsite) getNode().getUserObject();
		AbstractJob job = new AbstractJob(servicePrefix + "_CopyJobs"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				ResourceManager.getUser(vsite));
		AJOIdentifierEnumeration idEnum = jobIds.elements();
		int jobCount = 0;
		while (idEnum.hasMoreElements()) {
			AJOIdentifier id = (AJOIdentifier) idEnum.nextElement();
			CopyJob cj = new CopyJob("CopyJob", null, id);
			job.add(cj);
			jobCount++;
		}
		if (jobCount == 0) {
			return abstractJobs;
		}
		// submit the job in non-polling mode
		Reply roReply = nonPolling(job, vsite, ResourceManager.getUser(vsite));
		// now analyze outcome
		if (!(roReply instanceof RetrieveOutcomeReply)) {
			UserMessages.warning("Could not contact virtual site " + vsite);
		}
		else {
			// deserialize the Outcome Object
			Outcome outcome = (Outcome) new ObjectInputStream(
					new ByteArrayInputStream(((RetrieveOutcomeReply) roReply)
							.getOutcome())).readObject();
			if (outcome instanceof AbstractJob_Outcome) {
				// get AbstractJob object from outcome
				AbstractJob_Outcome aj_out = ConsignForm
						.convertFrom((AbstractJob_Outcome) outcome);
				OutcomeEnumeration oen = aj_out.getOutcomes();
				while (oen.hasMoreElements()) {
					Outcome next = oen.nextElement();
					if (next instanceof CopyJob_Outcome) {
						AbstractJob originalJob = ((CopyJob_Outcome) next)
								.getJob();
						// do not add service jobs
						String jobName = originalJob.getAJOId().getName();
						if (originalJob != null
								&& !jobName.startsWith(servicePrefix)) {
							abstractJobs.add(originalJob);
						}
					}
				}
			}
		}
		return abstractJobs;
	}

	private AJOIdentifierSet getJobIds() throws Exception {
		GetJobs gj = new GetJobs("GetJobs");
		Vsite vsite = (Vsite) getNode().getUserObject();
		User user = ResourceManager.getUser(vsite);
		// must construct an abstract job wrapper; consign it in non-polling
		// mode
		AbstractJob job = new AbstractJob(servicePrefix + "_GetJobs"
				+ ResourceManager.getNextObjectIdentifier(), null, null, vsite,
				user);
		job.add(gj);
		Reply roReply = nonPolling(job, vsite, user);
		// now analyze outcome
		if (!(roReply instanceof RetrieveOutcomeReply)) {
			UserMessages.warning("Could not contact virtual site " + vsite);
			return null;
		}
		else {
			// deserialize the Outcome Object
			Outcome outcome = (Outcome) new ObjectInputStream(
					new ByteArrayInputStream(((RetrieveOutcomeReply) roReply)
							.getOutcome())).readObject();
			if (outcome instanceof AbstractJob_Outcome) {
				AbstractJob_Outcome aj_out = ConsignForm
						.convertFrom((AbstractJob_Outcome) outcome);
				GetJobs_Outcome gjo = (GetJobs_Outcome) aj_out.getOutcome(gj
						.getId());
				return gjo.getJobs();
			}
		}
		return null;
	}

	/**
	 * Get Vector of jobs
	 * 
	 * @return jobs
	 */
	public Vector getKnownJobs() {
		return knownJobs;
	}

	private JobContainer reconstructContainer(AbstractJob job) {
		// build final container
		JobContainer container = JobContainer.decode(job.getExtraInformation());
		if (container != null) {
			container.setIdentifier(job.getId());
		}
		return container;
	}

	/**
	 * Start the Thread
	 */
	public void run() {
		try {
			getNode().setState(ContainerNode.STATE_UPDATING);
			// first get ids of jobs known to njs
			AJOIdentifierSet jobIds = getJobIds();
			// get copy of original AbstractJob objects from njs
			Vector abstractJobs = getAbstractJobs(jobIds);
			// reconstruct containers from AbstractJob objects
			knownJobs = new Vector();
			for (int i = 0; i < abstractJobs.size(); i++) {
				AbstractJob job = (AbstractJob) abstractJobs.elementAt(i);
				JobContainer container = reconstructContainer(job);
				// if container is null it could not be decoded
				if (container != null) {
					knownJobs.add(container);
				}
				else {
					// construct a dummy container for AJO's not submitted
					// by our client
					AjoContainer group = new AjoContainer();
					group.setIdentifier(job.getId());
					group.setName(job.getId().getName());
					group.setActionGroup(job);
					group.setVsite(job.getVsite());
					group.setUser(ResourceManager.getUser(job.getVsite()));
					knownJobs.add(group);
				}
			}
			VectorSorter.sort(knownJobs);
		}
		catch (Exception e) {
			logger.log(Level.SEVERE, "Getting AJO Identifiers for " + getNode()
					+ " failed.", e);
		}
		// notify all my observers on task end
		notifyObservers(this, getNode());
	}
}