/*
 * Copyright (c) 2000-2004, Intel Corporation All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Intel Corporation nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.pallas.unicore.resourcemanager;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Logger;
import org.unicore.Vsite;
import com.pallas.unicore.extensions.NamedResourceSet;
import com.pallas.unicore.extensions.Usite;
import com.pallas.unicore.utility.VectorSorter;

/**
 * Cache information about usites, vsites and resources
 * 
 * @author Ralf Ratering
 * @version $Id: ResourceCache.java,v 1.2 2004/05/27 15:53:55 ratering Exp $
 */
public class ResourceCache implements Serializable {

	private static Logger logger = Logger
			.getLogger("com.pallas.unicore.resourcemanager");
	final static long serialVersionUID = -4561301205740529479L;
	private final static String[] specialVsiteTypes = {"AFT", "SAFE"};
	private Hashtable resourceTable = new Hashtable();
	private Hashtable specialVsiteTable = new Hashtable();
	private Hashtable unavailableUsiteTable = new Hashtable();
	private Hashtable unavailableVsiteTable = new Hashtable();
	private Vector usites = new Vector();
	private Hashtable vsiteTable = new Hashtable();

	/**
	 * Constructor
	 */
	public ResourceCache() {
	}

	/**
	 * add resource set for a vsite
	 */
	public void addResourceSet(Vsite vsite, NamedResourceSet resourceSet) {
		resourceTable.put(ResourceManager.getUniqueVsiteIdentifier(vsite),
				resourceSet);
	}

	/**
	 * Adds a special type of vsite
	 */
	public void addSpecialVsite(Vsite vsite, String message) {
		logger.info(message);
		if (specialVsiteTable == null) {
			specialVsiteTable = new Hashtable();
		}
		specialVsiteTable.put(ResourceManager.getUniqueVsiteIdentifier(vsite),
				message);
	}

	/**
	 * Add unavailabe USite with corresponding error message
	 */
	public void addUnavailableUsite(Usite usite, String message) {
		logger.info(message);
		unavailableUsiteTable.put(usite, message);
	}

	/**
	 * Add unavailabe VSite with corresponding error message
	 */
	public void addUnavailableVsite(Vsite vsite, String message) {
		logger.info(message);
		unavailableVsiteTable.put(ResourceManager
				.getUniqueVsiteIdentifier(vsite), message);
	}

	/**
	 * Add vector of vsites for a usite
	 */
	public void addVsites(Usite usite, Vector vsites) {
		// sort vsites
		VectorSorter.sort(vsites);
		vsiteTable.put(usite, vsites);
	}

	/**
	 * remove all special vsites
	 */
	public void clearSpecialVsites() {
		specialVsiteTable.clear();
	}

	/**
	 * remove unavailable usites
	 */
	public void clearUnavailableUsites() {
		unavailableUsiteTable.clear();
	}

	/**
	 * remove unavailable vsites
	 */
	public void clearUnavailableVsites() {
		unavailableVsiteTable.clear();
	}

	/**
	 * This method is needed as long as we do not have a Vsite.equals() and
	 * Vsite.hashCode() method. It returns the Vsite from the cache matching the
	 * characteristics of the provided vsite, which may be a different object.
	 *  
	 */
	public Vsite getMatchingVsite(Vsite vsite) {
		Usite usite = null;
		for (int i = 0; i < usites.size(); i++) {
			usite = (Usite) usites.elementAt(i);
			Vector vsites = getVsites(usite);
			for (int j = 0; j < vsites.size(); j++) {
				Vsite next = (Vsite) vsites.elementAt(j);
				if (ResourceManager.getUniqueVsiteIdentifier(vsite).equals(
						ResourceManager.getUniqueVsiteIdentifier(next))) {
					return next;
				}
			}
		}
		return null;
	}

	/**
	 * Get resources for a specific VSite.
	 * 
	 * @param vsite
	 *            Vsite to get resources from
	 * @return ResourceSet containing all resources from the VSite
	 */
	public NamedResourceSet getResourceSet(Vsite vsite) {
		// if no resources have been loaded return default empty set
		if (vsite == null
				|| resourceTable.get(ResourceManager
						.getUniqueVsiteIdentifier(vsite)) == null) {
			return NamedResourceSet.getEmptyResourceSet();
		}
		return (NamedResourceSet) resourceTable.get(ResourceManager
				.getUniqueVsiteIdentifier(vsite));
	}

	/**
	 * Gets the Usite for a Vsite
	 * 
	 * @param vsite
	 *            The Vsite for which the Usite is asked for
	 * @return The Usite value
	 */
	public Usite getUsite(Vsite vsite) {
		Usite usite = null;
		for (int i = 0; i < usites.size(); i++) {
			usite = (Usite) usites.elementAt(i);
			Vector vsites = getVsites(usite);
			for (int j = 0; j < vsites.size(); j++) {
				Vsite next = (Vsite) vsites.elementAt(j);
				if (ResourceManager.getUniqueVsiteIdentifier(vsite).equals(
						ResourceManager.getUniqueVsiteIdentifier(next))) {
					return usite;
				}
			}
		}
		// this should only happen if no Usites exist
		return usite;
	}

	/**
	 * Gets the error message for a usite
	 */
	public String getUsiteErrorMessage(Usite usite) {
		Object message = unavailableUsiteTable.get(usite);
		if (message == null) {
			return null;
		}
		return (String) message;
	}

	/**
	 * Gets all usites
	 */
	public Vector getUsites() {
		return usites;
	}

	/**
	 * Get error message for a Vsite
	 */
	public String getVsiteErrorMessage(Vsite vsite) {
		Object message = unavailableVsiteTable.get(ResourceManager
				.getUniqueVsiteIdentifier(vsite));
		if (message == null) {
			return null;
		}
		return (String) message;
	}

	/**
	 * Get all vsites for a usite
	 */
	public Vector getVsites(Usite usite) {
		// if no vsites have been loaded return empty Vector
		if (vsiteTable.get(usite) == null) {
			return new Vector();
		}
		return (Vector) (vsiteTable.get(usite));
	}


	/**
	 * Checks if this Vsite can accept AJO's Special Vsites, like AFT and SAFE
	 * will appear on the gateway, but cannot handle AJO's
	 */
	private boolean isOnSpecialList(Vsite vsite) {
		for (int j = 0; j < specialVsiteTypes.length; j++) {
			if (vsite.getType().getDescription().equals(specialVsiteTypes[j])) {
				logger.info("Vsite " + vsite.getName() + " excluded");
				return true;
			}
		}
		return false;
	}

	/**
	 * Check if Vsite is a sepcial one
	 */
	public boolean isSpecialVsite(Vsite vsite) {
		if (specialVsiteTable == null) {
			specialVsiteTable = new Hashtable();
		}
		return specialVsiteTable.get(vsite) != null;
	}

	/**
	 * Check if usite is available
	 */
	public boolean isUsiteAvailable(Usite usite) {
		return unavailableUsiteTable.get(usite) == null;
	}

	/**
	 * Check if vsite is available
	 */
	public boolean isVsiteAvailable(Vsite vsite) {
		return unavailableVsiteTable.get(ResourceManager
				.getUniqueVsiteIdentifier(vsite)) == null;
	}

	/**
	 * Remove a Vector of outdated vsite entries from resource table
	 */
	private void removeResourceEntries(Vector oldVsites) {
		Enumeration vsiteEnum = resourceTable.keys();
		while (vsiteEnum.hasMoreElements()) {
			// Vsite vsite = (Vsite) vsiteEnum.nextElement();
			Object vsite = vsiteEnum.nextElement();
			if (oldVsites.contains(vsite)) {
				resourceTable.remove(vsite);
			}
		}
	}

	/**
	 * Dump cache contents
	 */
	public String toString() {
		String dump = "*** RESOURCE CACHE ***";
		dump += "\nUSITES: " + usites;
		dump += "\nUNAVAILABLE USITES: " + unavailableUsiteTable;
		dump += "\nVSITES: " + vsiteTable;
		dump += "\nUNAVAILABLE VSITES: " + unavailableVsiteTable;
		dump += "\nRESOURCES: " + resourceTable;
		return dump;
	}

	/**
	 * update usite list
	 */
	void updateUsites(Vector newUsites) {
		for (int i = 0; i < newUsites.size(); i++) {
			Usite newUsite = (Usite) newUsites.elementAt(i);
			boolean found = false;
			for (int j = 0; !found && j < usites.size(); j++) {
				Usite oldUsite = (Usite) usites.elementAt(j);
				if (newUsite.equals(oldUsite)) {
					//newUsites.setElementAt(oldUsite,
					// newUsites.indexOf(newUsite));
					usites.remove(oldUsite);
					// setting flag will exit loop
					found = true;
				}
			}
		}
		// remove remaining usites from vsite table and
		// corresponding vsites from resource table
		for (int i = 0; i < usites.size(); i++) {
			Usite oldUsite = (Usite) usites.elementAt(i);
			Vector oldVsites = (Vector) vsiteTable.get(oldUsite);
			// if vsites have been available remove them from
			// vsite and resource table
			if (oldVsites != null) {
				removeResourceEntries(oldVsites);
				vsiteTable.remove(oldUsite);
			}
		}
		// finally we have the new usites
		usites = newUsites;
		VectorSorter.sort(usites);
	}

	/**
	 * update vsites
	 */
	Vector updateVsites(Usite usite, Vsite[] newVsites) {
		Vector result = new Vector();
		// get old vsites from table if available
		Vector oldVsites = (Vector) vsiteTable.get(usite);
		for (int i = 0; i < newVsites.length; i++) {
			boolean found = false;
			// if we had vsites for our usite before,
			// use old ones if they still exist
			if (oldVsites != null) {
				for (int j = 0; !found && j < oldVsites.size(); j++) {
					Vsite oldVsite = (Vsite) oldVsites.elementAt(j);
					// check also if a vsite changed its type
					if (ResourceManager
							.getUniqueVsiteIdentifier(newVsites[i])
							.equals(
									ResourceManager
											.getUniqueVsiteIdentifier(oldVsite))
							&& newVsites[i].getType()
									.equals(oldVsite.getType())) {
						//result.add(oldVsite);
						// removing existing vsites will
						// outdated ones in the vector
						oldVsites.remove(oldVsite);
						found = true;
					}
				}
			}
			//            if(!found) {
			result.add(newVsites[i]);
			//            }
		}
		// result now contains the current vsites and oldVsites
		// the outdated ones
		// loop over new vsites and check AJO compatibility
		for (int i = 0; i < result.size(); i++) {
			Vsite vsite = (Vsite) result.elementAt(i);
			if (isOnSpecialList(vsite)) {
				addSpecialVsite(vsite, "Vsite type excluded: "
						+ vsite.getType().getDescription());
				result.remove(vsite);
			}
		}
		// remove old vsites from resource table, if available
		if (oldVsites != null) {
			removeResourceEntries(oldVsites);
		}
		return result;
	}

	/**
	 * Check if usite exists
	 */
	public boolean usiteExists(Usite usite) {
		return usites.contains(usite);
	}

	/**
	 * check if vsite exists
	 */
	public boolean vsiteExists(Vsite vsite) {
		return resourceTable.containsKey(ResourceManager
				.getUniqueVsiteIdentifier(vsite));
	}
}