package org.unicore;

import java.security.cert.X509Certificate;
import java.security.cert.CertificateFactory;

import java.io.InputStream;
import java.io.ByteArrayInputStream;

/**
 * Ulogin identifies a Unicore client.
 * <p>
 * Clients can be either users, through a JPA/JMC,
 * or NJSs, when consigning sub-AJOs to other NJSs.
 * <p>
 * Identification is done through X509 certificates.
 * 
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9 (AJO 1 in other forms)
 *
 * @version Ulogin.java 1.7 09/09/02 
 * 
 **/

public class Ulogin extends Unicore {

    /**
     * Create an uninitialised Ulogin.
     *
     **/
    public Ulogin() {
    }

    /**
     * Create a new Ulogin. 
     * <p>
     * Uses the given certificate as
     * the identity of this Ulogin. It also creates an encoded
     * form of this certificate that is suitable for transmitting
     * to other Unicore servers. Note that the certificate itself
     * is not transmitted.
     * <p>
     * The encoded form of the certificate contains the data
     * in a form that is independant of the class used to represent
     * the certificate. It is obtained by calling ((X509Certificate)certificate).getEncoded()
     *
     * @throws java.security.cert.CertificateException There was a problem encoding the certificate.
     *
     * @deprecated AJO 4.0 Create with a full certificate chain
     *
     **/
    public Ulogin(X509Certificate certificate) throws java.security.cert.CertificateException {
        this(new X509Certificate[] {certificate});
    }

    /**
     * Create a new Ulogin.
     * <p>
     * Uses the given certificate chain to establish the identity of this Ulogin
     * (end certificate is a position 0)
     *
     **/
    public Ulogin(X509Certificate[] certificate_chain) throws java.security.cert.CertificateException {
        setCertificateChain(certificate_chain);
    }

    /**
     * Get the end entity certificate used to identify a client.
     * 
     **/
    public X509Certificate getCertificate() {
        return certificate_chain[0];
    }

    private transient X509Certificate[] certificate_chain;
    /**
     * Get the certificate chain used to identify a client.
     *
     **/
    public X509Certificate[] getCertificateChain() {
        return certificate_chain;
    }

    /**
     * Set a new certificate for this Ulogin. 
     * <p>
     *
     * The given certificate is used as
     * the identity of this Ulogin. This also creates an encoded
     * form of this certificate that is suitable for transmitting
     * to other Unicore servers. Note that the certificate itself
     * is not transmitted.
     * <p>
     * The encoded form of the certificate contains the data
     * in a form that is independant of the class used to represent
     * the certificate. It is obtained by calling ((X509Certificate)certificate).getEncoded()
     *
     * @throws java.security.cert.CertificateException There was a problem encoding the certificate.
     *
     * @deprecated AJO 4.0 Use setCertificateChain() to set the complete chain
     *
     **/
    public void setCertificate(X509Certificate certificate) throws java.security.cert.CertificateException {
        setCertificateChain(new X509Certificate[] {certificate});
    }

    /**
     * Set a new certificate chain for this Ulogin.
     *
     **/
    public void setCertificateChain(X509Certificate[] certificate_chain) throws java.security.cert.CertificateException {
        this.certificate_chain = certificate_chain;
        encoded_chain = new byte[certificate_chain.length][];
        for(int i = 0; i < certificate_chain.length; i++) {
            encoded_chain[i] = certificate_chain[i].getEncoded();
        }
        encoded = encoded_chain[0];  // compatibility with previous AJOs
    }

    /**
     * Return a name for this Ulogin.
     * <p>
     * The name is derived from the given X509 certificate as
     * the name of the certificate's subject's distinguished name.
     *
     **/
    public String getName() {
        return certificate_chain[0].getSubjectDN().getName();
    }

    private byte[] encoded;
    /**
     * Return the encoded form of the end entity certificate.
     *
     **/
    private byte[] getEncoded() {return encoded_chain[0];}

    private byte[][] encoded_chain;
    /**
     * Return the encoded form of the certificate chain
     *
     **/
    private byte[][] getEncodedChain() {return encoded_chain;}

    /**
     * This method returns equals for a Ulogin if
     * the end entity certificates are the same.
     *
     **/
    public boolean equals(Object in) {
        boolean result = false;
        if(in == null) {
            result = false;
        }
        else {
            result = in.getClass() ==  this.getClass() &&
                     getCertificate().equals( ((Ulogin)in).getCertificate());
        }

        return result;
    }

    /**
     * Ensure that there is an instance for "certificate_chain"
     * after deserialisation.
     *
     **/
    private void readObject(java.io.ObjectInputStream stream) throws java.io.IOException, ClassNotFoundException {
        
        stream.defaultReadObject();      // Do normal deserialisation first

        // Now instantiate local certificates using local classes

        // What have we read in? Either an old form with just the
        // end entity certificate or a chain (not necessarily full)
        if(encoded_chain == null) {
            // Just the certificate
            certificate_chain = new X509Certificate[] {decodeCert(encoded)};
            encoded_chain = new byte[][]{encoded};
        }
        else {
            // A chain
            certificate_chain = new X509Certificate[encoded_chain.length];
            for(int i = 0; i < encoded_chain.length; i++) {
                certificate_chain[i] = decodeCert(encoded_chain[i]);
            }
            encoded = encoded_chain[0];
        }
    }

    private X509Certificate decodeCert(byte[] encoded) throws java.io.IOException, ClassNotFoundException {
        try {
            if(cf == null) cf = CertificateFactory.getInstance("X.509");
            return (X509Certificate)cf.generateCertificate(new ByteArrayInputStream(encoded));
        }
        catch(java.security.cert.CertificateException cex) {
            // Rethrow this information as readObject signature asks
            throw new java.io.IOException("Ulogin certificate instantiation error: "+cex.getMessage());
        }
    }

    private transient static CertificateFactory cf;

    static final long serialVersionUID = -3800442287289509005L;

}
//
// Change history:
//    03 Sep 2002 SvdB Takes certificate chains.
//    16 Nov 2000 SvdB removed references to IAIK
//    10 Sep 2000 SvdB Returns a stab at a name for the Ulogin
//    30 Apr 2000 SvdB V3.0 changes 
//     6 Apr 2000 SvdB added readObject to decode encoded 
//     6 Apr 2000 SvdB changed to reflect better understanding
//                     of the how Certficates work 
//    15 Mar 2000 SvdB Transfer as provider neutral, do as X509 
//     4 Feb 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
