package org.unicore;

/**
 * A Vsite is a collection of resources on which a 
 * Job runs. 
 * <p>
 * A client can get a description of the Vsites in a
 * Usite by sending the Gateway a {@link org.unicore.upl.ListVsites}
 * request. The Gateway will return a list of the Vsites under
 * its control. Each instance of Vsite will contain further
 * information about the Vsite. This information includes the
 * type of the Vsite, i.e. if it an NJS processing AbstractJobs or
 * a SAFE server processing a different protocol. A Vsite
 * instance also contains information about the specification
 * of the requests supported by the Vsite. This specification
 * information does not include the UPL layer. We assume that the UPL
 * specification is constant.
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 1 in other forms)
 *
 * @version Vsite.java 1.13 11/29/02 
 * 
 **/
public final class Vsite extends Unicore {

    /**
     * A Vsite.Reference contains information about a way to contact a Vsite.
     *
     * @since AJO 4.0
     *
     **/
    public static class Reference extends Unicore {
        static final long serialVersionUID = 400;

        private String handle;
        private String address;

        public Reference(String handle, String address) {
            this.handle = handle;
            this.address = address;
        }

        public Reference() {
        }

        /**
         * Return the handle of the Vsite for which this
         * Reference contains contact information
         *
         **/
        public String getHandle() {return handle;}

        /**
         * Return the address through which the Vsite the
         * Vsite can be contacted (a URL)
         *
         **/
        public String getAddress() {return address;}

    }

    public static class Type extends Unicore {
        static final long serialVersionUID = 3496753410568194152L;

        /**
         * This Vsite is an NJS providing AJO execution services.
         *
         **/
        public static final Type NJS = new Type("NJS");

        /**
         * @deprecated Don't use this (define your own subtype)
         *
         **/
        public static final Type BROKER = new Type("BROKER");

        /**
         * This Vsite is a SAFE providing file transfer services.
         *
         **/
        public static final Type SAFE = new Type("SAFE");

        /**
         * An AJO containing an instance of this can be rerouted to other
         * Vsites if the NJS wants to.
         *
         **/
        public static final Type MUTABLE= new Type("MUTABLE");

        /**
         * Create a new Type. 
         * <p>
         * Note that this is 
         * publically available so that the range
         * of supported types can be extended by
         * agreement between the clients and sites.
         *
         **/
        public Type(String description) {
            this.description = description;
        }

        /**
         * Create a new NJS Type
         *
         **/
        public Type() {
            this.description = "NJS";
        }

        private String description;
        /**
         * Get the decsription of the type
         *
         **/
        public String getDescription() {return description;}

        /**
         * Types are equal if their descriptions are equal.
         * <p>
         * Need to do carefully as these cross JVM vboundaries
         * through serialisation and so "==" will fail even
         * on statics.
         *
         **/
        public boolean equals(Object o) {
            try {
                return ( o.getClass() == getClass() &&
                         getDescription().equals(((Type)o).getDescription()) );
            }
            catch(NullPointerException nex) {
                return false;
            }
        }

        public int hashCode() {
            return getDescription().hashCode();
        }
    }

    /**
     * Create an uninitialised Vsite.
     *
     **/
    public Vsite() {
        this(null);
    }

    /**
     * Create a Vsite.
     *
     * @deprecated AJO 4.0 The address (aka reference) for a Vsite will eventually be external to a Vsite instance
     *
     * @param address Where to contact the Gateway for this Vsite.
     * @param name The name of the Vsite.
     *
     **/
    public Vsite(String address, String name) {
        this(name);
        setAddress(address);
    }

    /**
     * Create a Vsite.
     *
     * @since AJO 4.0
     *
     * @param name The name of the Vsite
     *
     **/
    public Vsite(String name) {
        setName(name);
        this.type = Type.NJS;
    }

    /**
     * Return the URL of the Unicore Gateway to the Vsite. 
     *
     * @deprecated AJO 4.0 Use an external mapper (temporarily Vsite.Reference can be used)
     *
     **/
    public String getAddress() {
        if(reference == null) {
            return null;
        }
        else {
            return reference.getAddress();
        }
    }

    /**
     * Set the URL of the Unicore Gateway to the Vsite. 
     *
     * @deprecated AJO 4.0 Use an external mapper (temporarily VsiteReference can be used)
     *
     **/
    public void setAddress(String address) {
        setReference(new Vsite.Reference(getHandle(),address));
    }

    private String name;
    /**
     * Return the name of the Vsite.
     * <p>
     * This is used to identify the Vsite to users and
     * to Gateways (so that Gateways supporting multiple
     * Vsites can direct the UPL Request to the correct Vsite).
     *
     **/
    public String getName() {
        return name;
    }

    /**
     * Set the name of the Vsite.
     * <p>
     * The pragmatic test for equality {@link #equals} uses a Vsite's name.
     * The following convention should be used to ensure that the equality test
     * works:
     * <p>
     * A Vsite name is: <Usite name>_<Vsite name>
     * <p>
     * Where <Usite name> is unique and <Vsite name> is unique eithin a Usite.
     * 
     **/
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Return the handle for the Vsite.
     * <p>
     * 
     * @since AJO 4.0
     *
     **/
    public String getHandle() {return name;}

    private Vsite.Reference reference;
    /**
     * Set the Reference to the Vsite. 
     *
     * @since AJO 4.0
     *
     * @deprecated AJO 4.0 In a future version of the AJO the reference will be moved out
     *                     of Vsite since the validity of a particular reference will depend
     *                     on the circumstances of the user of the Reference. For example,
     *                     a site could provide an internal Gateway just for local users.
     *                     At this point Vsite References will be obtained through some
     *                     external mapping process.
     *
     **/
    public void setReference(Vsite.Reference reference) {
        this.reference = reference;
        this.name = reference.getHandle();
    }

    /**
     * @since AJO 4.0
     *
     * @deprecated AJO 4.0 Vsite references will eventually be obtained from some
     *                     mapping process that is aware of local circumstances etc
     *
     **/
    public Vsite.Reference getReference() {return reference;}

    /**
     * Returns true if the test object is the same Vsite as this one.
     * <p>
     * A particular user can find the same AJO at two Vsites if and only if the two Vsites are the
     * same (equal). This suggests an "operational" test for Vsite equality: consign a job (AJO1)
     * to Vsite1. AJO1 contains a sub AJO (AJO2) which is consigned to Vsite2. AJO2 contains a 
     * {@link org.unicore.ajo.GetJobs}. If the Outcome of the GetJobs contains the AJOIdentifier
     * of AJO1, then Vsite1 is equal to Vsite2.
     * <p>
     * This test is not pracitcal so this implementation of equals returns true if the 
     * Vsites have the same name (handle) and we need to establish a convention to ensure that
     * this has the same results as the test above.
     *
     **/
    public boolean equals(Object test) {
        if(test instanceof Vsite) {
            if(name == null) {
                return ((Vsite)test).getName() == null;
            }
            else {
                return name.equals(((Vsite)test).name);
            }
        }
        else {
            return false; // null or not a Vsite 
        }
    }

    public int hashCode() {
        if(name == null) {
            return 0;
        }
        else {
            return name.hashCode();
        }
    }
    

    /**
     * Return the name and type of the Vsite as a String
     *
     **/
    public String toString() {
        return getName() +" <"+getType().getDescription()+">";
    }


    private Type type;
    /**
     * Return the type of the Vsite. 
     * <p>
     * A site may have
     * a number of different servers offering different
     * services. For example NJSs to execute AJOs or
     * SAFE servers for file transfer. The type allows
     * clients to differentiate between these and so
     * adjust their requests to those appropriate to
     * the server type.
     *
     **/
    public Type getType() {return type;}

    /**
     * Set the type of the Vsite. 
     *
     **/
    public void setType(Type type) {this.type = type;}


    private String ajo_specification_title;
    /**
     * Return the title of the specification of the payload
     * of UPL requests and replies handled by this Vsite.
     *
     * The payload of a UPL request or reply is the information
     * contained in the byte arrays returned by
     * {@link org.unicore.upl.ConsignJob#getAJO} and
     * {@link org.unicore.upl.RetrieveOutcomeReply#getOutcome}.
     *
     * This information can be used to ensure that
     * the client and server are using compatible versions
     * of the software for the type of the Vsite.
     *
     * The format of the title and version information should
     * be the same as that used in java.lang.Package.
     *
     * The Vsite instance does not contain the implementation
     * information from the java.lang.Package class as the 
     * intention of this information is to ensure that the
     * client and server are using compatible versions and
     * not to provide a full definition of the software used
     * by the Vsite.
     *
     **/
    public String getAJOSpecificationTitle() {return ajo_specification_title;}

    /**
     * @deprecated AJO 4 I should learn to spell and proofread?
     *
     **/
    public String getAJOSpecficationTitle() {return ajo_specification_title;}

    /**
     * Set the title of the specification of the payload
     * of UPL requests and replies handled by this Vsite.
     *
     **/
    public void setAJOSpecificationTitle(String title) {ajo_specification_title = title;}

    /**
     * @deprecated AJO 4 I should learn to spell and proofread?
     *
     **/
    public void setAJOSpecficationTitle(String title) {ajo_specification_title = title;}

    private String ajo_specification_version;

    /**
     * Return the version of the specification of the payload
     * of UPL requests and replies handled by this Vsite.
     *
     * The version string should be compatible with java.lang.Package
     *  and be suitable as an argument for the isCompatibleWith() method.
     *
     **/
    public String getAJOSpecificationVersion() {return ajo_specification_version;}

    /**
     * @deprecated AJO 4 I should learn to spell and proofread?
     *
     **/
    public String getAJOSpecficationVersion() {return ajo_specification_version;}

    /**
     * Set the version of the specification of the payload
     * of UPL requests and replies handled by this Vsite.
     *
     **/
    public void setAJOSpecificationVersion(String version) {ajo_specification_version= version;}

    /**
     * @deprecated AJO 4 I should learn to spell and proofread?
     *
     **/
    public void setAJOSpecficationVersion(String version) {ajo_specification_version= version;}

    static final long serialVersionUID = -1918310711746266075L;

    // Compatibility with stored previous versions
    private String address;

    private void readObject(java.io.ObjectInputStream stream) throws java.io.IOException, ClassNotFoundException {
        stream.defaultReadObject(); // Do normal deserialisation first

        if(reference == null) {
            if(address != null) setReference(new Reference(name,address));
        }

        address = null;

    }

}
//
// Change history:
//          29 Nov 2002 SvdB Empty constructors for Type and Reference
//          17 Jul 2002 SvdB Added handle, reference and equals
//          23 Apr 2002 SvdB Added MUTABLE Vsites
//                           removed BROKER Vsite - now a resource (oops, keep as deprecated so V3 Gateways OK)
//          30 Jan 2001 SvdB Added AJO specification information
//          16 Nov 2000 SvdB Added Type information
//          30 Apr 2000 SvdB Moved to Vsite for 3.0 
//           4 Feb 2000 SvdB Created for V2.9 as marker 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
