package org.unicore.ajo;

import org.unicore.Unicore;
import org.unicore.AAIdentifier;

/**
 * An AbstractAction describes work to be performed by Unicore. 
 * <p>
 * All definitions of work are types of AbstractAction. The unit
 * of sending a work request to a Vsite through a {@link org.unicore.upl.ConsignJob} request
 * is an {@link AbstractJob} which is itself an AbstractAction but
 * also contains a graph of AbstractActions that defines the work
 * to be performed.
 *
 * @author M. Drescher (Pallas)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version AbstractAction.java 1.11 02/10/03 
 *
 **/
public abstract class AbstractAction extends Unicore {

    static final long serialVersionUID = -1503460335910313926L;

    protected AbstractAction() {
        this("", null);
    }

    /**
     * Create an uninitialised AbstractAction.
     *
     **/
    protected AbstractAction(String name) {
        this(name, null);
    }

    /**
     * Create an AbstractAction.
     *
     * @param name The name of the AbstractAction
     * @param extra_information Client supplied extra information about the AbstractAction
     *
     **/
    protected AbstractAction(String name, byte[] extra_information ) {
        id = new AAIdentifier(name);
        this.extra_information = extra_information;
    }

    /**
     * Get the name of this AbstractAction.
     * <p>
     * The name can be used by programmers and users to 
     * identify a particular AbstractAction. It is a convenience field as 
     * the identity of the instance is contained in the
     * {@link org.unicore.AAIdentifier}.
     *
     **/
    public String getName() {return id.getName();}

    /**
     * Change the name of this AbstractAction.
     *
     **/
    public void setName(String name) {id.setName(name);}

    private AAIdentifier id;
    /**
     * Return the Identifier if the AbstractAction.
     *
     * The Identifier is used to identify an instance of an AbstractAction
     * within a Unicore system. This is needed as instances of AbstractAction
     * move between different Virtual Machines (and Java object identity is only unique
     * within a VM). For example, the Identifier is used by a client to request the Outcome
     * of executing an AbstractJob.
     *
     **/
    public AAIdentifier getId() { return id; }

    /**
     * Set the Identifier of the AbstractAction.
     *
     **/
    public void setId(AAIdentifier id) {this.id = id;}

    /**
     * Equality of AbstractActions is determined by equality of their Identifiers.
     * <p>
     * The Identifier is used to identify the AbstractAction so we
     * need to provide an implementation of equals that uses
     * this value. Thus an instance of AbstractAction will
     * be equal to itself across different Virtual Machines.
     *
     **/
    public boolean equals(Object in) {
        return( in.getClass() ==  this.getClass() &&
                getId().equals(((AbstractAction)in).getId()) 
              );
    }

    /**
     * Return a hash code based on the Identifier value.
     * <p>
     * Follows from over-riding equals.
     *
     **/
    public int hashCode() {return getId().hashCode();}

    private byte[] extra_information;
    /**
     * Return the extra information about this AbstractAction.
     * <p>
     * A holder for extra information used by the creator
     * of the AbstractAction. Can be used, for example, by the
     * JPA to keep information about the icon placement
     * so that consistent display is available to mobile users.
     *
     **/
    public byte[] getExtraInformation() {return extra_information;}

    /**
     * Set the extra information about this AbstractAction.
     *
     **/
    public void setExtraInformation(byte[] extra_information) { 
        this.extra_information = extra_information;
    }

    private String notes;
    /**
     * Return the notes added to this AbstractAction.
     * <p>
     * The notes string is a space for clients to add information
     * about the purpose of an AbstractAction e.g. to allow a
     * user to add notes about the values in a particular AbstractAction
     * or for the client software to note why it has added
     * certain AbstractActions that are not visible to the user.
     *
     **/
    public String getNotes() {return notes;}

    /**
     * Set some notes to this AbstractAction.
     *
     **/
    public void setNotes(String notes) {this.notes = notes;}

    /**
     * Return a name for (UPL) directories relating to this AbstractAction.
     *<p>
     * When streaming data the UPL places the all the Portfolios
     * in the Outcome of an AbstractAction in a directory.
     * This method ensures that
     * both ends of a transfer use the same name for the directory.
     *
     **/
    public String getUPLDirectoryName() {
        return "AA"+Integer.toHexString(id.getValue());
    }

    private java.util.Date termination_time;

    /**
     * @return The latest time that a task is valid
     *
     * @since AJO 4.0
     *
     **/
    public java.util.Date getTerminationTime() {return termination_time;}

    /**
     * Set the latest time that a task is valid.
     * <p>
     * If the termination time passes the AbstractAction is {@link Control}.CANCELled.
     * <p>
     * The termination time can be extended by {@link ExtendTerminationTime} up to a maximum.
     *
     * @since AJO 4.0
     * 
     **/
    public void setTerminationTime(java.util.Date termination_time) {this.termination_time = termination_time;}

    private long max_extension;
    /**
     * @return The maximum time that the validity of the AbstractAction can be extended
     *
     * @since AJO 4.0
     *
     **/
    public long getMaxExtension() {return max_extension;}

    /**
     * Set the maximum time that the validity of the AbstractAction can be extended.
     * <p>
     * If this value is exceeded, then {@link ExtendTerminationTime} will fail.
     *
     * @since AJO 4.0
     *
     **/
    public void setMaxExtension(long max_extension) {this.max_extension = max_extension;}

    private boolean propagate_successful_if_not_successful = false;

    public void setPropagateSuccessfulIfNotSuccessful(boolean b) {propagate_successful_if_not_successful = b;}

    /**
     * Return the action that will be taken following execution of the AbstractAction
     * to propagate the final {@link org.unicore.outcome.AbstractActionStatus} to 
     * dependant successor AbstractActions.
     * <p>
     * If this is true, then the status propagated to dependant actions following execution
     * is {@link org.unicore.outcome.AbstractActionStatus}.SUCCESSFUL.
     * <p>
     * If this is false (the default), then the actual final status of the AbstractAction is
     * propagated to dependants.
     *
     * This has no effect if the AbstractAction is not executed so that {@link org.unicore.outcome.AbstractActionStatus}.NEVER_TAKEN
     * and {@link org.unicore.outcome.AbstractActionStatus}.NEVER_RUN are always propagated to dependants
     * unchanged.
     * <p>
     * Note that if the AbstractAction is the target of a {@link Control}.ABORT or a {@link Control}.CANCEL, then
     * its final status is {@link org.unicore.outcome.AbstractActionStatus}.KILLED and the propagation of this value
     * <em>is</em> subject to modification by this flag. This also applies if the AbstractAction is aborted when its
     * termination time passes.
     *
     * @since AJO 4.0
     *
     **/
    public boolean propagateSuccessfulIfNotSuccessful() {return propagate_successful_if_not_successful;}

}
//
// Change history:
//          22 Jan 2002 SvdB Reinterpret beyond Termination Time to mean CANCEL, not ABORT
//          03 Sep 2002 SvdB Added propagate_successful_if_not_successful
//          23 Apr 2002 SvdB Added termination time, max extension
//          18 Mar 2002 SvdB V4 no arg constructor
//          16 Nov 2000 SvdB added notes field, moved getUPLDirectoryNAme from AAIdentifier
//           3 May 2000 SvdB V3.0 
//           9 Feb 2000 SvdB added set methods (for AJO "deconsignform"ing) 
//          19 Apr 1999 SvdB Added setExtraInformation for MD 
//          29 Jan 1999 SvdB Used getid in equals
//           7 Dec 1998 SvdB Second release to Unicore
//           5 Nov 1998 SvdB Added byte[] for extra information
//          12 Jun 1998 SvdB Initial release to Unicore
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
