package org.unicore.ajo;

import org.unicore.sets.ResourceSet;

import org.unicore.resources.Resource;

/**
 * AbstractTasks perform work on the target systems in a Vsite. 
 * <p>
 * They are incarnated and executed on the site's systems, for example
 * under control of a BSS or by a Storage Server. AbstractTasks
 * consume resources and so contain a request for the resources
 * that they need.
 *
 * Example AbstractTasks are those for transferring files or
 * executing scripts.
 *
 * @author M. Drescher (Pallas)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version AbstractTask.java 1.8 04/17/03 
 *
 **/
public abstract class AbstractTask extends AbstractAction {

    static final long serialVersionUID = -3280241124323645432L;

    /**
     * Create a new AbstractTask.
     * <p>
     * @param name The name of the AbstractTask
     * @param extra_information Client supplied extra information about the AbstractTask
     * @param resources Resources needed to execute the AbstractTask
     *
     **/
    protected AbstractTask( String name,
                            byte[] extra_information,
                            ResourceSet resources
                       ) {
        super(name,extra_information);

        this.resources = resources;
    }

    private ResourceSet resources;
    /**
     * Return the resources that will be needed by the incarnation of the AbstractTask. 
     * <p>
     * Where the resource request includes a quantity, this quantity
     * is a job total <em>not</em> per processor e.g. for
     * 10 seconds on 20 nodes the client should request 200 seconds.
     * not 20.
     * <p>
     * In some cases this resource description is used to
     * determine where a task should be executed, example the
     * location of the files for any {@link FileAction}.
     * <p>
     * The client can find out about a Vsite's resources by sending
     * a {@link GetResourceDescription}. The Vsite will respond with a 
     * description of the supported resources.
     *
     * @since AJO 3
     *
     **/
    public ResourceSet getResources() {return resources;}

    /**
     * Change the resources needed. 
     * <p>
     * This overwrites any previous values.
     *
     * @since AJO 3
     *
     **/
    public void setResources(ResourceSet resources) {
        this.resources = resources;
    }

    /**
     * Add a resource to the requested resources.
     *
     * @since AJO 3
     *
     **/
    public void addResource(Resource resource) {
        if(resource != null) {
            if(resources == null) resources = new ResourceSet();
            resources.add(resource);
        }
    }

    /**
     * Remove a resource from the requested resources.
     *
     * @since AJO 4.0
     *
     **/
    public void removeResource(Resource resource) {
        if(resources != null) resources.remove(resource);
    }

}
//
// Change history:
//          23 Jul 2002 SvdB Added removeResource, addResource no longer checks if in
//          11 Apr 2002 SvdB Removed getResourceLimits
//           9 Jun 2000 SvdB deprecated "limits" naming
//           5 May 2000 SvdB V3.0
//           7 Dec 1998 SvdB Second release to Unicore
//          12 Jun 1998 SvdB Initial release to Unicore
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
