package org.unicore.ajo;

/**
 * ConditionalActions support If-Then-Else style processing in an AJO. 
 * <p>
 * Subtypes of ConditionalAction provide a test condition to select which of
 * the branches will be executed. The selected branch is executed and the not selected
 * branch is given a final status of {@link org.unicore.outcome.AbstractActionStatus}.NEVER_TAKEN.
 * <p>
 * Note that the branch AbstractActions are not part of the ConditionalAction's
 * parent ActionGroup. However, execution of the selected branch will take place
 * within the root-AJO's Uspace.
 * <p>
 * The ConditionalAction executes if its target action has completed execution, otherwise it fails
 * with a final status of FAILED_IN_INCARNATION.
 * If the ConditionalAction is executed, then its final status is the final status of the selected branch (or SUCCESSFUL if
 * the selected branch was null).
 *
 * @author D. Snelling (fecit)
 *
 * @since AJO 3.2
 *
 * @version ConditionalAction.java 1.8 10/30/02
 *
 * @see org.unicore.outcome.ConditionalAction_Outcome
 *
 **/
public abstract class ConditionalAction extends AbstractAction {

    static final long serialVersionUID = 0;

    /**
     * Create a new ConditionalAction.
     * <p>
     * @param name The name of the ConditionalAction
     * @param extra_information Client supplied extra information about the ConditionalAction
     * @param true_branch The action executed if the expression evaluated to true.
     * @param fail_branch The action executed if the expression evaluated to false.
     *
     **/
    protected ConditionalAction( String name,
                                 byte[] extra_information,
                                 AbstractAction true_branch,
                                 AbstractAction false_branch
                                ) {
        super(name,extra_information);
        this.true_branch = true_branch;
        this.false_branch = false_branch;
    }

    protected AbstractAction true_branch;

    /**
     * Return the action to be executed if the expression evaluates to true.
     * <p>
     * Can be null.
     *
     **/
    public AbstractAction getTrueBranch() {return true_branch;}

    /**
     * Set the true branch.
     *
     **/
    public void setTrueBranch (AbstractAction true_branch) {this.true_branch = true_branch;}

    protected AbstractAction false_branch;

    /**
     * Return the action to be executed if the expression evaluates to false.
     * <p>
     * Can be null.
     *
     **/
    public AbstractAction getFalseBranch() {return false_branch;}

    /**
     * Set the false branch.
     *
     **/
    public void setFalseBranch (AbstractAction false_branch) {this.false_branch = false_branch;}

}
//
// Change history:
//          30 Jan 2001 DFS Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
