package org.unicore.ajo;

import java.util.Enumeration;

import org.unicore.Unicore;

/**
 *
 * The parent class for type safe C-like enum types. 
 * <p>
 * Enums are
 * derived from this Class and, as far as possible, the derived
 * classes provide only the instance information and this parent
 * class provides the functionality. <p>
 *
 *  Enums have the following characteristics:
 * <ul>
 *     <li> they are equal across VMs (provided the "value" is the same)
 *     <li> all the instances of a derived Enum can be enumerated
 *     <li> they can be used as indicies into arrays (if "value" is sequential)
 * </ul>
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 1
 *
 * @version Enum.java 1.6 09/09/02 
 *
 **/
public abstract class Enum extends Unicore {

    static final long serialVersionUID = -7228111278596073273L;

 // NOTE This implementation of Enums is not particularly good. All
 //      instances of all child classes are in the same list and so
 //      serialisation serialissed the lot. This has been fixed -
 //      temporarily? - by making the list and next transient. This
 //      works because the methods that use list and next are static
 //      and so do not access any deserialised instances.
 //
 // NOTE (May 2000 SvdB) Deserialised instances do not point to the
 //      statics for the VM. Is this a problem or just untidy? Can
 //      it be fixed by a readObject() trick? Is it worth it?
 //


    /**
     * The value that makes an instance unique
     * within its type. This value may be the same
     * in different types but needs to be different
     * for each instance of a subclass.
     *
     **/
    private int value;

    /**
     * For convenience we give each instance
     * a name.
     *
     **/
    private String name;

   /**
    * We keep a linked list of all the instances of all
    * the derived classes so that we can enumerate Enums.
    *
    */
   transient Enum next;

   /**
    *  Root of the list.
    *
    */
   static transient Enum first = null;


    /**
     * Create a new Enum value.
     * <p>
     * The only constructor and the only means to
     * setting an instance's values. The intention is
     * that this is only called by child classes constructors.
     *
     * @param aValue The value for this instance.
     * @param aName The name for this instance.
     *
     **/
    protected Enum(int aValue, String aName) {
        value = aValue;
        name = aName;

        next = first;
        first = this;
    }

    /**
     * Return the unique value. 
     * <p>
     * This acts as an index variable if
     * subclasses are setup as 0,1,2,3...
     *
     **/
    public int getValue() { return value; }

    /**
     * Return the name.
     *
     **/
    public String getName() { return name; }

    public String toString() {return name;}

    /**
     * Get equality across VMs.
     * <p>
     * We use getClass() here to test that the derived types
     * are the same. It is in the superclass to prevent having
     * to repeat code in the derived classes.
     **/

    public boolean equals(Object in) {
        return ( in.getClass() ==  this.getClass() &&
                 this.value == ((Enum) in).value );
    }

    /**
     * A hash code for an instance of an Enum.
     * <p>
     * Returns the value.
     *
     **/
    public int hashCode() {return value;}

    /**
     * Enumerate the elements of the <b>sub</b> class.
     * <p>
     * @param enum_class The subclass to be enumerated.
     * @return an Enumeration
     *
     **/
    public static Enumeration elements(Class enum_class) {

      // Create an enumeration on the Enums of the calling class only.
      // That is, this is called by elements() in the derived classes
      // with an (any) instance of the derived class as test. The
      // enumeration is set up to loop through the linked list of
      // all Enum instances but only return those of the derived class
      // (by comparing the classes through getClass)

      return new Enumerate( enum_class );
    }

}

    class Enumerate implements Enumeration {
            private Enum curr;
            private Class wanted;

            Enumerate(Class in) {
                wanted = in;

                // move to first
                curr = Enum.first;
                while( curr != null && (curr.getClass() != wanted) ) {
                    curr = curr.next;
                }
            }

            public boolean hasMoreElements() { return curr != null; }

            public Object nextElement() {
                Enum old = curr;
                curr = curr.next;
                while( curr != null && (curr.getClass() != wanted) ) {
                    curr = curr.next;
                }
                return old;
            }
    }
//
// Change history:
//          12 Oct 2000 SvdB  Put in a correct hash code for across VM values
//           5 May 2000 SvdB  V3.0 
//           7 Dec 1998 SvdB  Do not serialize the list
//           7 Dec 1998 SvdB  Second release to Unicore
//          12 Jun 1998 SvdB  Initial release to Unicore
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
