package org.unicore.ajo;

import org.unicore.PortfolioIdentifier;
import org.unicore.AJOIdentifier;

import org.unicore.sets.ResourceSet;

/**
 * Copy a Portfolio from the local Uspace to the remote Uspace.
 * <p>
 * The source Portfolio is in the Uspace of the containing AJO, the path
 * to the Portfolio is given in the instance of {@link org.unicore.resources.USpace}
 * (default root of the Uspace).  The destination of the Portfolio is given
 * by the instance of {@link org.unicore.resources.Spool} or 
 * {@link org.unicore.resources.AlternativeUspace} (default is to Spool the Portfolio).
 * <p>
 * Portfolios are always created hidden in the alternative Uspace and so any
 * path component is ignored.
 * <p>
 * If the target Portfolio or its containing Uspace
 * does not exist, then this task will create it and
 * its Uspace (if necessary) 
 *<p>
 * The remote Uspace is on the same Vsite.
 *<p>
 * If the Target Portfolio identifier is null the NJS will create an identifier
 * when the PutPortfolio is executed. This identifier will be unique. The identifier
 * used is returned in the Outcome. One use of this is to Spool a Portfolio
 * during each iteration of a RepeatGroup.
 * <p>
 * Copies of directories are recursive.
 *
 * @see org.unicore.outcome.PutPortfolio_Outcome
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3 (AJO 2 as ReplacePortfolio)
 *
 * @version PutPortfolio.java 1.13 09/18/02
 **/

public final class PutPortfolio extends FileTransfer {

    static final long serialVersionUID = 7758674493952761251L;

    /**
     * Create a new PutPortfolio.
     * <p>
     * @param name The name of the PutPortfolio
     * @param extra_information Client supplied extra information about the PutPortfolio
     * @param resources Resources needed to execute the PutPortfolio
     * @param target The destination Portfolio
     * @param target_uspace The destination Uspace
     * @param source The source Portfolio
     * @param target_must_exist Policy on creating target
     *
     * @deprecated AJO 4.0 No target_must_exist or target_uspace
     *
     **/
    public PutPortfolio( String name,
                         byte[] extra_information,
                         ResourceSet resource_limits,
                         PortfolioIdentifier target,
                         AJOIdentifier target_uspace,
                         Portfolio source,
                         boolean target_must_exist
                               ) {

        super(name,extra_information,resource_limits);

        this.target = target;
        this.target_uspace = target_uspace;
        this.source = source;
        this.target_must_exist = target_must_exist;
    }

    /**
     * Create a new PutPortfolio.
     * <p>
     * @param name The name of the PutPortfolio
     * @param resources Resources needed to execute the PutPortfolio
     * @param target The destination Portfolio
     * @param target_uspace The destination Uspace
     * @param source The source Portfolio
     * @param target_must_exist Policy on creating target
     *
     * @deprecated AJO 4.0 No target_must_exist or target_uspace
     *
     **/
    public PutPortfolio( String name,
                         ResourceSet resource_limits,
                         PortfolioIdentifier target_portfolio,
                         AJOIdentifier target_uspace,
                         Portfolio source,
                         boolean target_must_exist
                               ) {

        this(name,(byte[]) null,resource_limits, target_portfolio, target_uspace, source, target_must_exist);
    }

    /**
     * Create a new PutPortfolio.
     * <p>
     * @param name The name of the PutPortfolio
     * @param extra_information Client supplied extra information about the PutPortfolio
     * @param resources Resources needed to execute the PutPortfolio
     * @param overwrite Whether existing files cane be overwriien when teh Portfolio is revealed.
     * @param target The destination Portfolio
     * @param source The source Portfolio
     *
     **/
    public PutPortfolio( String name,
                         byte[] extra_information,
                         ResourceSet resource_limits,
                         boolean overwrite,
                         PortfolioIdentifier target,
                         Portfolio source
                               ) {

        super(name,extra_information,resource_limits);

        this.target = target;
        this.source = source;
        this.overwrite = overwrite;
    }

    /**
     * Create a new PutPortfolio.
     * <p>
     * @param name The name of the PutPortfolio
     * @param resources Resources needed to execute the PutPortfolio
     * @param overwrite Whether existing files cane be overwriien when teh Portfolio is revealed.
     * @param target The destination Portfolio
     * @param target_uspace The destination Uspace
     * @param source The source Portfolio
     *
     **/
    public PutPortfolio( String name,
                         ResourceSet resource_limits,
                         boolean overwrite,
                         PortfolioIdentifier target_portfolio,
                         Portfolio source
                               ) {

        this(name,(byte[]) null,resource_limits, overwrite, target_portfolio, source);
    }

    /**
     * Create a new PutPortfolio.
     * <p>
     * @param name The name of the PutPortfolio
     *
     **/
    public PutPortfolio(String name) {
        this(name,(byte[]) null, (ResourceSet) null, true, (PortfolioIdentifier) null, (Portfolio) null);
    }

    public PutPortfolio() {
        this("");
    }

    private PortfolioIdentifier target;
    /**
     * Return the identifier of the Portfolio to replace.
     *
     **/
    public PortfolioIdentifier getTarget() {
        return target;
    }

    /**
     * Set the identifier of the Portfolio to replace.
     *
     **/
    public void setTarget(PortfolioIdentifier target) {
        this.target = target;
    }

    private AJOIdentifier target_uspace;
    /**
     * Return the destination Uspace for the copy.
     *
     * @deprecated AJO 4.0 Uspace is set by the AlternativeUspace resource
     *
     **/
    public AJOIdentifier getTargetUspace() {return target_uspace;}

    /**
     * Set the destination Uspace for the copy.
     *
     * @deprecated AJO 4.0 Uspace is set by the AlternativeUspace resource
     *
     **/
    public void setTargetUspace(AJOIdentifier target_uspace) {this.target_uspace = target_uspace;}

    private Portfolio source;
    /**
     * Get the Portfolio to copy.
     *
     **/
    public Portfolio getSource() {return source;}

    /**
     * Set the Portfolio to copy.
     *
     **/
    public void setSource(Portfolio source) {this.source = source;}

    private boolean target_must_exist;
    /**
     * Get policy on creating new remote Portfolio.
     * <p>
     * If this is true, then the task succeeds only if the 
     * target Portfolio exists. Otherwise, the task creates
     * a new Portfolio and, if necessary, the Uspace to hold it.
     *
     * @deprecated AJO 4.0 Portfolio always created (hidden), Uspace created is necessary
     *
     **/
    public boolean targetMustExist() {return target_must_exist;}

    /**
     * Set policy on creating new remote Portfolio.
     *
     * @deprecated AJO 4.0 Portfolio always created (hidden), Uspace created is necessary
     *
     **/
    public void setTargetMustExist(boolean target_must_exist) {
        this.target_must_exist = target_must_exist;
    }

    boolean overwrite;
    /**
     * Overwrite existing destination files?
     * <p>
     * If no overwriting is allowed (this method returns false), then revealing the files
     * will fail if it would overwrite an existing file. Note that the overwrite test
     * is applied when the Portfolio is revealed and not when the PutPortfolio is executed.
     * If file in the Portfolio is a
     * directory, then the test will be applied to each file in the directory (that
     * is, writing files into existing directories is always allowed).
     * <p>
     * Overwriting existing files or writing into existing directories is always
     * subject to permissions on the existing files or directories and the task
     * will fail if there is no suitable write permission.
     *
     **/
    public boolean willOverwrite() {return overwrite;}

    /**
     * Set overwrite policy.
     *
     **/
    public void setOverwrite(boolean overwrite) {this.overwrite = overwrite;}

}
//
// Change log:
//          15 Jul 2002 SvdB New semantics on null Target
//          18 Dec 2002 SvdB V4 no arg constructor, etc
//          12 Jun 2000 SvdB More consistent naming?
//           5 May 2000 SvdB V3.0 
//          30 Apr 2000 SvdB Renamed from ReplacePortfolio for V3.0 
//           7 Dec 1998 SvdB Second release to Unicore 
//          27 Nov 1998 SvdB SvdB Added must exist boolean 
//           5 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
