package org.unicore.ajo;

import java.util.*;
import org.unicore.Unicore;
import org.unicore.ajo.ResourceDependency;

/** 
 * A ResourceDAG is an partially ordered collection of TaskResource nodes.
 *
 * The ResourceSets are ordered in a way that is defined by the
 * {@link org.unicore.ajo.ResourceDependency}relationships.
 * (The implementation is a directed acyclic graph).
 * It represents a group of abstract tasks and corresponding resource requrements
 * grouped in a Unicore job with the relationships between them.
 *
 * @author Antoan Bekele
 * @author M. Drescher (Pallas)
 * @author D. Snelling (fecit)
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 3.6
 *
 * @version TaskResourceDAG.java 1.4 10/14/02
 *
 **/
public class TaskResourceDAG extends Unicore{

    static final long serialVersionUID = -1062272996095563272L;

    private Vector nodes;
    private Vector edges;
    private Vector successors;

    /**
     * Thrown when a dependency could not be added to a TaskResourceDAG. The dependency
     * was either invalid (predecessor or successor TaskResource are null or
     * not both in the ActionGroup) or would introduce cycles into the TaskResourceDAG.
     *
     **/
    public static class InvalidResourceDependencyException extends Exception {
        static final long serialVersionUID = 400L;
        public InvalidResourceDependencyException() {
            super();
        }
        public InvalidResourceDependencyException(String message) {
            super(message);
        }
    }

    /**
     * Creates a new TaskResourceDAG.
     *
     **/
    public TaskResourceDAG()
    {
        nodes = new Vector();
        edges = new Vector();
        successors = new Vector();
    }

    /**
     * Adds a TaskResource node to this DAG.
     *
     * @param taskResource Task paired with its Resource request
     *
     **/
    public void add(TaskResource taskResource) {
        nodes.addElement(taskResource);
    }

    /**
     * Defines a dependency between two TaskResource nodes of this dag.
     * 
     * @param link ResourceDependency that connects two TaskResource nodes.
     *
     * @deprecated AJO 4 Use addDependency(ResourceDependency link) instead, <em>or</em>
     *             always check that the new dependency is valid and will not introduce cycles by
     *             calling canAdd(ResourceDependency link) first.
     **/
    public void addResourceDependency(ResourceDependency link) {
        edges.addElement(link);

        // Add the successor to the Vector, Vector is a bag
        // and so multiple entries represent the number of dependencies
        successors.addElement(link.getSuccessor());
    }

    /**
     * Defines a dependency between two TaskResource nodes of this dag.
     *
     * @throws InvalidResourceDependencyException The new dependency is invalid or would create a cycle in the TaskResourceDAG
     **/
    public void addDependency(ResourceDependency link) throws InvalidResourceDependencyException {
        if(!canAdd(link)) {
            throw new InvalidResourceDependencyException("The new dependency is invalid or would create a cycle in the TaskResourceDAG");
        }

        edges.addElement(link);

        // Add the successor to the Vector, Vector is a bag
        // and so multiple entries represent the number of dependencies
        successors.addElement(link.getSuccessor());
    }

    /**
     * Add a dependency between the two TaskResources (adding the TaskResources if necessary)
     *
     * @throws InvalidResourceDependencyException The new dependency would create a cycle in the TaskResourceDAG
     *
     **/
    public void addDependency(TaskResource predecessor, TaskResource successor) throws InvalidResourceDependencyException {

        ResourceDependency link = new ResourceDependency(predecessor,successor);

        try {
            addDependency(link);
        }
        catch(InvalidResourceDependencyException idex) {
            // Why'd it go wrong?
            if(predecessor == null) throw new InvalidResourceDependencyException("Predecessor TaskResource is null");
            if(successor   == null) throw new InvalidResourceDependencyException("Successor TaskResource is null");

            if(!isIn(predecessor)) add(predecessor);
            if(!isIn(successor))   add(successor);

            try {
                addDependency(link);
            }
            catch(InvalidResourceDependencyException idexidex) {
                throw new InvalidResourceDependencyException("The new dependency would create a cycle in the TaskResource");
            }

        }
    }

    /**
     * Tests if a dependency between TaskResource nodes represented by ResourceDependency
     * will introduce a cycle in this dag.
     *
     * @param link ResourceDependency
     * @return boolean value representing the validity of adding this ResourceDependency.
     *
     **/
    public boolean canAdd(ResourceDependency link) {

        TaskResource pred = link.getPredecessor();
        TaskResource succ = link.getSuccessor();

        if( pred == null || succ == null ) {
            return false;
        }
        else if( pred.equals(succ) ) {
            return false;
        }
        else if ( !( isIn(pred) && isIn(succ) ) ) {
            return false;
        }
        else {
            // Does this ResourceDependency introduce any cycles?
            // We **know** that there are no cycles in the DAG
            // now so we only need to look for an existing path from
            // succ to pred.

            CycleFinder c = new CycleFinder(pred);
            return !c.find(succ);
        }
    }

    class CycleFinder {
        Object start; // if we return to this we have a cycle
        ResourceDependency d[];

        public CycleFinder(Object s) {
            start = s;
            // Copy ResourceDependencies out (cleaner/more efficient)
            d = new ResourceDependency[edges.size()];
            edges.copyInto(d);
        }

        public boolean find(Object from) {

            boolean cycle_found = false;

            for(int i=0; i < d.length; i++) {
                if(d[i].getPredecessor().equals(from)) {
                    if(d[i].getSuccessor().equals(start)) {
                        cycle_found = true;
                    }
                    else {
                        cycle_found = find(d[i].getSuccessor());
                    }
                    if(cycle_found) break;
                }
            }
            return cycle_found;
        }
    }



    /**
     * Get an enumeration over all the <tt>TaskResource nodes</tt>s in the
     * group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Returns a, possibly empty, <tt>Enumeration</tt>, of all
     *             <tt>TaskResource nodes</tt>s contained in the group.
     * <p>
     *
     * @return Enumeration of TaskResource objects
     **/
    public Enumeration getTaskResources() {
        return nodes.elements();
    }

    /**
     * Get an enumeration over all the <tt>ResourceDependencies</tt>s in the
     * group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Returns a, possibly empty, <tt>Enumeration</tt> of all
     *             <tt>ResourceDependencies</tt>s contained in the group.
     * <p>
     * @return Enumeration of ResourceDependency objects
     **/
    public Enumeration getResourceDependencies() {
        return edges.elements();
    }

    /**
     * Test if an <tt>TaskResource node</tt> is in the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Return true iff the <tt>TaskResource</tt> is
     *             in the group.
     * <p>
     * @param obj
     *
     **/
    public boolean isIn(Object obj) {
        return nodes.contains(obj);
    }

    /**
     * Test if an <tt>ResourceDependency</tt> is in the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> Return true iff the <tt>ResourceDependency</tt> is
     *              in the group.
     * <p>
     *
     **/
    public boolean isIn(ResourceDependency link) {
        return edges.contains(link);
    }

    /**
     * Remove an <tt>TaskResource</tt> from the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The <tt>TaskResource</tt> is not in
     *              the group and any ResourceDependencies that relied transitively
     *              on that <tt>TaskResource</tt> have been replaced
     *              with direct ResourceDependencies.
     * <p>
     **/
    public void remove(Object action) {

        if( nodes.removeElement(action) ) {
            // The TaskResource was in the DAG
            // We now need to find out which ResourceDependencies it was
            // involved in

            Vector in_pred = new Vector(edges.size());
            Vector in_succ = new Vector(edges.size());

            for( int i=edges.size()-1; i >= 0 ; i--) {
                // ... loop backwards as we are removing elements

                ResourceDependency d = (ResourceDependency) edges.elementAt(i);

                if(d.getPredecessor().equals(action)) {
                    in_pred.addElement(d);
                    remove(d); // remove the old ResourceDependency from group
                }
                else if (d.getSuccessor().equals(action)) {
                    in_succ.addElement(d);
                    remove(d); // remove the old ResourceDependency from group
                }

            }

            // Now add the new ResourceDependencies
            for(int s=0; s < in_succ.size(); s++) {
                for(int p=0; p < in_pred.size(); p++) {
                    ResourceDependency d = new ResourceDependency(
                    ((ResourceDependency) in_succ.elementAt(s)).getPredecessor(),
                    ((ResourceDependency) in_pred.elementAt(p)).getSuccessor()
                    );
                    if(!isIn(d)) addResourceDependency(d);


                }
            }

        }

        // Note. If the TaskResource was not in the DAG, then
        // the precondition on addDependency ensures that there
        // are no depnedencies involving this TaskResource. So
        // do nothing here.

    }

    /**
     * Remove an <tt>ResourceDependency</tt> from the group.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The ResourceDependency is not in the group.
     * <p>
     **/


    public void remove(ResourceDependency link) {

        edges.removeElement(link);

        successors.removeElement(link.getSuccessor());
    }

    /**
     * Clear the group of all TaskResource(s) and, since without TaskResource(s) they
     * have no meaning, all ResourceDependencies.
     * <p>
     * <b>Pre:</b>  True.
     * <br>
     * <b>Post:</b> The group is empty.
     * <p>
     **/
    public void removeAllTaskResources() {
        nodes.removeAllElements();
        removeAllResourceDependencies();
    }

    /**
     * Clear the group of all ResourceDependencies.
     *<p>
     * <b>Pre:</b>  True.
     *<br>
     * <b>Post:</b> There are no ResourceDependencies in the group (TaskResource(s) remain).
     *<p>
     **/
    public void removeAllResourceDependencies() {
        edges.removeAllElements();
        successors.removeAllElements();
    }

    /**
     * Return the TaskResources that are ready to be processed,
     * that is they are not a successor in any dependency. A 
     * DAG can be walked by using this method on a copy
     * and removing those visited.
     * <p>
     * This method considers only its own DAG so it returns 
     * any child TaskResources.
     * <p>
     * <b>Pre:</b> True.
     * <br>
     * <b>Post:</b> The enumeration over any ready actions
     * <p>
     **/
    public Enumeration ready() {

        // We have maintained an accurate list of all successor
        // Actions in successors. 

        Vector result = new Vector();

        Enumeration e = getTaskResources();
        while(e.hasMoreElements()) {
            Object node = e.nextElement();
            if( !successors.contains(node) ) {
                // This action is not a successor,
                // so must be ready
                result.addElement(node);
            }
        }

        return result.elements();
    }
}
//
// Change history:
//          12 Dec 2001 DFS Created as a copy of ActionGroup
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
