package org.unicore.ajo;

import org.unicore.outcome.AbstractActionStatus;
import org.unicore.ajo.AbstractAction;
import org.unicore.Unicore;

/**
 *
 * Indication of when something should happen relative to an AbstractAction
 * entering a specified state (also the start of the Unix epoch).
 * 
 * @author Sven van den Berghe (fecit)
 *
 * @since AJO 4.0
 *
 * @version Time.java 1.7 11/01/02
 *
 **/
public class Time extends Unicore {

    /**
     * Create an instance of Time that evaluates to
     * the start of the epoch.
     *
     **/
    public Time() {
    }

    /**
     * Create a Time that evaluates to the number of milliseconds
     * since the start of the epoch (the same as java.util.Date(long date)).
     *
     **/
    public Time(long time) {
        setOffset(time);
    }

    /**
     * Create a new Time that evaluates to the last time that the target
     * AbstractAction entered the state (with an offset).
     *
     **/
    public Time(AbstractAction target, AbstractActionStatus status, long offset) {
        setTarget(target);
        setStatus(status);
        setOffset(offset);
    }

    private AbstractAction target;
    /**
     * @return The target AbstractAction
     *
     **/
    public AbstractAction getTarget() {return target;}

    /**
     * Set the target AbstractAction. The status of this AbstractAction will be
     * queried when the containing expression is evaluated.
     * <p>
     * The executing NJS must be able to get the status of the target AbstractAction
     * (the target must be on the NJS and owned by the same User). If the target
     * is not available, then the expression evaluation fails and the execution of the 
     * containing task fails.
     * <p>
     * If target is null, then the time returned is  offset milliseconds from the
     * start of the epoch (January 1, 1970, 00:00:00 GMT)
     *
     **/
    public void setTarget(AbstractAction target) {
        this.target = target;
    }

    private AbstractActionStatus status = AbstractActionStatus.PENDING;
    /**
     * Set the target status, when this Time is evaluated the time that the target AbstactAction
     * went into this status is returned. If the target entereed this state more then once the
     * latest time is returned. If the target has never entered this state, then the evaluation
     * fails and the execution of the containing task fails. The default value is {@link org.unicore.outcome.AbstractActionStatus}.PENDING
     * 
     **/
    public void setStatus(AbstractActionStatus status) {if(status != null) this.status = status;}

    /**
     * @return The target status
     *
     **/
    public AbstractActionStatus getStatus() {return status;}


    private long offset;
    /**
     * Set the offset from the time that the target entered the specified state.
     * <p>
     * This is in milliseconds. The offset must be positive.
     *
     **/
    public void setOffset(long offset) {this.offset = offset;}

    /**
     * @return The offset
     *
     **/
    public long getOffset() {return offset;}

    public String toString() {
        if(target != null) {
            return "Action "+Integer.toHexString(target.getId().getValue())+" status "+status+" offset "+offset;
        }
        else {
            return "Offset " + (new java.util.Date(offset));
        }
    }

}
//
// Change history:
//          30 Oct 2002 SvdB Target is action, not identifier (resetIdentifiers?)
//          17 Apr 2002 SvdB Created for V4.0 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
