package org.unicore.utility;

import org.unicore.ajo.*;
import org.unicore.outcome.*;
import org.unicore.sets.*;
import org.unicore.upl.*;

import org.unicore.Vsite;
import org.unicore.User;

import java.io.*;
import java.security.*;
import java.util.*;

/**
 * Converts AJOs and Outcomes to and from a form suitable for consigning.
 * <p>
 * AJOs and Outcomes are transported in the UPL through two levels of serialisation.
 * The UPL Objects,e.g. {@link org.unicore.upl.ConsignJob}, are transported
 * using the usual serialisation. However, when deserialised UPL objects contain
 * a byte array which is the serialisation of the AJO or Outcome. This double serialisation
 * means that intermediate servers, such as Gateways, which deserialise the UPL
 * objects do not have to deserialise the AJO or Outcome object and so do not
 * need to have the full AJO classes available to them.
 * <p>
 * In addition AJOs are are transmitted with a signature which mean that
 * NJSs can check that the AJOs have not been tampered with in transit.
 * <p>
 * The static methods in this class convert to and from the consign form
 * serialising the AJOs and Outcomes or deserialising them.
 *
 * @see org.unicore.ajo.AbstractJob#setConsignForm
 * @see org.unicore.ajo.AbstractJob#setSignature
 * @see org.unicore.outcome.AbstractJob_Outcome#setAnonOutcomes
 * @see org.unicore.upl.ConsignJob
 * @see org.unicore.upl.RetrieveOutcomeReply
 *
 * @author S. van den Berghe (fecit)
 *
 * @since AJO 2.9
 *
 * @version ConsignForm.java 1.16 10/10/02 
 * 
 **/
public final class ConsignForm {

    /**
     * Returns an initialised Signature instance ready to
     * sign some bytes.
     * <p>
     * This is used by {@link ConsignForm#convertTo} every time that
     * it needs to compute the signature of an AJO or sub-AJO.
     * <p>
     * <em>This method is supplied as a work-araound for a problem with
     * the Sun JSSE 1.0.2 Cryptographic Provider which meant that an
     * instance of Signature could only be used once.</em>
     *
     **/
    public static interface SignatureFactory {
        /**
         * Return an instance of Signature that can sign some bytes.
         * Returns null if an instance could not be created.
         *
         **/
        public Signature getInstance();
    }

    /**
     * Return an AJO ready to be consigned with the data serialised
     * and signed.
     * <p>
     * This takes an AJO with all fields in edtiable form and
     * returns a copy AJO with the consigning fields set (that is
     * the byte array for the serialised form and the byte array
     * for the signature) and empty editable fields.
     * <p>
     * Works recursively through the sub-AJOs converting them first.
     *
     * @param ajo The top-level AbstractJob to be converted
     * @param signer A signer object initialised with the user's private key ready to
     *               sign the serialised AJO and using MD5withRSA.
     *
     * @return A newly allocated AJO instance in consign form 
     *
     **/
    public static AbstractJob convertTo(AbstractJob ajo, SignatureFactory signer)  throws InvalidClassException,
                                                 NotSerializableException,
                                                 IOException, 
                                                 SignatureException {

        AbstractJob ajo_copy = null;

        {  // set scope for memory management

            // get a copy of the AJO
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(bos);
            oos.writeObject(ajo);
            oos.close();

            try {
                ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(bos.toByteArray()));

                bos = null; // memory management

                ajo_copy = (AbstractJob) ois.readObject();
            }
            catch(Exception ex) {
                // We have just created the stream so none of the exceptions
                // will be thrown so just swallow them
                // What about IO?
            }

        }  // end of memory management scope

        toConsignForm(ajo_copy, signer);    

        return ajo_copy;

    }


    private static void scanActionGroup(ActionGroup me, SignatureFactory signer) throws SignatureException {
        // Convert all children
        Enumeration actions = me.getActions();
        while(actions.hasMoreElements()) {
            AbstractAction child = (AbstractAction)actions.nextElement();
            if(child instanceof AbstractJob) {
                toConsignForm((AbstractJob)child,signer);
            }
            else if(child instanceof ConditionalAction) {

                ConditionalAction child_ca = (ConditionalAction) child;

                if(child_ca.getTrueBranch() instanceof AbstractJob) {
                    toConsignForm((AbstractJob)child_ca.getTrueBranch(),  signer);  
                }
                else if(child_ca.getTrueBranch() instanceof ActionGroup) {
                    scanActionGroup((ActionGroup)child_ca.getTrueBranch(),  signer);  
                }

                if(child_ca.getFalseBranch() instanceof AbstractJob) {
                    toConsignForm((AbstractJob)child_ca.getFalseBranch(),  signer);  
                }
                else if(child_ca.getFalseBranch() instanceof ActionGroup) {
                    scanActionGroup((ActionGroup)child_ca.getFalseBranch(),  signer);  
                }

            }
            else if(child instanceof ActionGroup) {
                scanActionGroup((ActionGroup)child, signer);
            }
        }

    }

    private static void toConsignForm(AbstractJob me, SignatureFactory signer) throws SignatureException {

        // Make sure all descendant AJOs are done first
        scanActionGroup(me,signer);

        // Convert self
        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            ObjectOutputStream oos = new ObjectOutputStream(bos);

            // Write the AJO specification version to the output bytes
            oos.writeObject( Package.getPackage("org.unicore").getSpecificationVersion() );

            // Write a time stamp to the output bytes
            oos.writeObject(new java.util.Date());

            // Do not want to transport old rubbish
            me.setConsignForm(null);
            me.setSignature(null);

            // This redundantly writes Id, Vsite, endorser to the byte stream
            // can live with this? (Must as the NJS can then safely
            // deserialise?)
            oos.writeObject(me);
            oos.close();

            // Memory management: can clear the AJOs editable DAG as
            // we have a copy in the bos
            me.removeAllActions();          // Clear the editable DAG

            me.setConsignForm(bos.toByteArray());
        }
        catch(Exception ex) {
            // Should not get any as we serialised/deserialised for a copy
        }

        Signature signature = signer.getInstance();
        if(signature == null) {
            throw new SignatureException("ConsignForm.convertTo() SignatureFactory did not return an instance.");
        }

        signature.update(me.getConsignForm());
        me.setSignature(signature.sign());

        // and clear the fields
        // No - needed for sub-AJOs me.setId((null);
        me.setExtraInformation(null);
        me.setSSO(null);
        // No - needed for sub-AJOs me.setVsite(null);
        // No - needed for sub-AJOs me.setEndorser(null);

        // Note that this does not clear any fields created by sub-classes
        // of AJO (e.g. Idiomatics) so that they are sent twice. This
        // is safe (see below)  but redundant

    }

    /**
     * Extracts an editable AJO from the serialised byte array in
     * the ConsignJob request. 
     * <p>
     * This does not verify the signature. 
     * <p>
     * This call converts a single level of AJO and
     * is thus suitable for NJSs.
     *
     * @return A newly allocated deserialised AJO
     *
     * @deprecated AJO 4.0 This does not provide a way to get at the time stamp and AJO specification
     *                     information of the deserialised AJO. Use {@link ConsignForm.AJO}
     *
     **/
    public static AbstractJob convertFrom(ConsignJob original) throws ClassNotFoundException,
                                                               InvalidClassException,
                                                               StreamCorruptedException,
                                                               OptionalDataException,
                                                               IOException {
        return (new ConsignForm.AJO(original)).getAJO();
    }


    /**
     * Extracts an editable AJO from the serialised byte array in
     * the AJO in place. 
     * <p>
     * This does no verification of the signature. 
     * <p>
     * This call recursively converts all levels level of AJO and
     * is thus suitable for clients. Returns the AbstractJob
     * with the editable fields filled in (to preserve references
     * in each AJO)
     * <p>
     * Clears the serialised byte array
     *
     * @return The original AJO with the editable fields set to the deserialisation
     *
     * @deprecated AJO 4.0 This does not provide a way to get at the time stamp and AJO specification
     *                     information of the deserialised AJO. Use {@link ConsignForm.AJO}
     *
     **/
    public static AbstractJob convertFrom(AbstractJob original) throws ClassNotFoundException,
                                                                  InvalidClassException,
                                                                  StreamCorruptedException,
                                                                  OptionalDataException,
                                                                  IOException {
        return (new ConsignForm.AJO(original)).getAJO();
    }



    /**
     * Convert the child Outcomes to a transmittal form.
     * <p>
     * This should be used by NJSs before
     * transmitting outcomes of child AbstractActions.
     * This returns an object which needs serialisation to be placed
     * into a {@link org.unicore.upl.RetrieveOutcomeReply}. This extra
     * level is needed to make child-AJO's status available to the 
     * consigning NJS while protecting the consigning NJS from classes
     * known only to the executing NJS.
     *
     * @return A newly allocated AbstractJob_Outcome with the anon forms set
     *         (and no editable Outcomes)
     *
     **/
    public static AbstractJob_Outcome convertTo(AbstractJob_Outcome ajoo) throws InvalidClassException,
                                                                                 NotSerializableException,
                                                                                 IOException {

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ObjectOutputStream oos = new ObjectOutputStream(bos);

        // Loop through the Actions of the AJO for each fetching
        // its Outcome from the AJO Outcome and serialising
        OutcomeEnumeration oe = ajoo.getOutcomes();
        while(oe.hasMoreElements()) {
            oos.writeObject(oe.nextElement());
        }

        oos.close();

        AbstractJob_Outcome result = new AbstractJob_Outcome(null,null);

        result.setId(ajoo.getId());
        result.setStatus(ajoo.getStatus());
        result.setReason(ajoo.getReason());
        result.setLog(ajoo.getLog());
        result.setStatusHistory(ajoo.getStatusHistory());

        result.setAnonOutcomes(bos.toByteArray());

        return result;
    }
    /**
     * Converts the child Outcomes from the transmittal form to a fully
     * deserialised instance. 
     * <p>
     * This should be used by JPAs before
     * trying to access any outcomes of child AbstractActions and
     * after the deserialisation of the Outcome in {@link org.unicore.upl.RetrieveOutcomeReply}.
     *
     * @return The original Outcome with the deserialised child Outcomes set
     *
     **/
    public static AbstractJob_Outcome convertFrom(AbstractJob_Outcome original) throws ClassNotFoundException,
                                                                  InvalidClassException,
                                                                  StreamCorruptedException,
                                                                  OptionalDataException,
                                                                  IOException {
        byte[] annon_outcomes = original.getAnonOutcomes();

        if(annon_outcomes==null) return original;

        original.setAnonOutcomes((byte[])null); // here coz the call clears the unsered ones

        ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(annon_outcomes));

        // The container had the non-anon cleared so set them from the deserialised 
        try {
            while(true) {
                original.addOutcome( doConvertFrom( (Outcome) ois.readObject() ) );
            }
        }
        catch(EOFException eex) {
            // OK, assume all read 
        }

        return original;

    }

    private static Outcome doConvertFrom(Outcome original) throws ClassNotFoundException,
                                                                  InvalidClassException,
                                                                  StreamCorruptedException,
                                                                  OptionalDataException,
                                                                  IOException {
        if(original instanceof AbstractJob_Outcome) {
            return convertFrom((AbstractJob_Outcome)original);
        }
        else if(original instanceof RepeatGroup_Outcome) {
            // Do all outcomes for all iterations
            ActionGroup_OutcomeEnumeration agoe = ((RepeatGroup_Outcome)original).getOutcomeList().elements();
            while(agoe.hasMoreElements()) {
                doConvertFrom(agoe.nextElement());
            }
        }
        else if(original instanceof ActionGroup_Outcome) {
            OutcomeEnumeration oe = ((ActionGroup_Outcome)original).getOutcomes();
            while(oe.hasMoreElements()) {
                doConvertFrom(oe.nextElement());
            }
        }
        else if(original instanceof ConditionalAction_Outcome) {
            doConvertFrom(((ConditionalAction_Outcome)original).getTakenOutcome());   // not_taken needs no work
        }
        else if(original instanceof CopyActionOutcome_Outcome) {
            doConvertFrom(((CopyActionOutcome_Outcome)original).getOutcome());
        }
        else {
            // everything else, nothing to do (including null)
        }

        return original;

    }


    /**
     * Return the creating AJO specification version.
     * <p>
     * The creating AJO specification version was set in the AJO's consign form when {@link #convertTo}
     * was last called (this can be used in java.lang.Package.isCompatibleWith() tests)
     * <p>
     * This method should be used if it is suspected that the AJO version is incomptaible (i.e. after
     * one of the ConsignForm.AJO constructors have thrown an exception).
     *
     **/
    public String getAJOSpecificationVersion(AbstractJob original) throws IOException {
        try {
            return (String) (new ObjectInputStream( new ByteArrayInputStream(original.getConsignForm()))).readObject();
        }
        catch(Exception ex) {
            throw new IOException("Errors reading AJO specification version from AJO consign form: "+ex);
        }
    }

                                                                
    /**
     * Class to get at the fields from a ConsignJob: the AJO plus its time stamp and
     * originating AJO specification version
     *
     * @since AJO 4.0
     *
     **/
    public static class AJO {

        private Date time_stamp;

        /**
         * Return the time stamp for this AJO.
         * <p>
         * The time stamp was set in the AJO's consign form when {@link #convertTo}
         * was last called.
         *
         **/
        public Date getTimeStamp() {return time_stamp;}

        private String ajo_spec;

        /**
         * Return the creating AJO specification version.
         * <p>
         * The creating AJO specification version was set in the AJO's consign form when {@link #convertTo}
         * was last called (this can be used in java.lang.Package.isCompatibleWith() tests)
         *
         **/
        public String getAJOSpecificationVersion() {return ajo_spec;}

        private AbstractJob ajo;

        public AbstractJob getAJO() {return ajo;}

        /**
         * Extracts an editable AJO from the serialised byte array in
         * the ConsignJob request. 
         * <p>
         * This does not verify the signature. 
         * <p>
         * This call converts a single level of AJO and
         * is thus suitable for NJSs.
         *
         **/
        public AJO(ConsignJob original) throws ClassNotFoundException,
                                                InvalidClassException,
                                                StreamCorruptedException,
                                                OptionalDataException,
                                                IOException {

            ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(original.getAJO()));

            try {
                ajo_spec = (String) ois.readObject();
            }
            catch(Exception ex) {
                throw new IOException("Errors reading AJO specification version from AJO consign form: "+ex);
            }

            try {
                time_stamp = (java.util.Date) ois.readObject();
            }
            catch(Exception ex) {
                throw new IOException("Errors reading date stamp from AJO consign form: "+ex);
            }

            ajo = (AbstractJob) ois.readObject();

            // Note that as this returns a new instance it safely sets Idiomatic
            // fields to the values from the signed byte array and not to the
            // copy sent plain
        }

        /**
         * Extracts an editable AJO from the serialised byte array in
         * the AJO in place.
         * <p>
         * This does no verification of the signature. 
         * <p>
         * This call recursively converts all levels level of AJO and
         * is thus suitable for clients. Returns the AbstractJob
         * with the editable fields filled in (to preserve references
         * in each AJO)
         * <p>
         * Clears the serialised byte array
         *
         **/
        public AJO(AbstractJob original) throws ClassNotFoundException,
                                                InvalidClassException,
                                                StreamCorruptedException,
                                                OptionalDataException,
                                                IOException {
            AbstractJob deserialised = null;

            // Derserialise me (if I look like I need it)
            byte[] serialised = original.getConsignForm();
            if(serialised == null) {
                // No serialised form, assume already OK
            }
            else {
                ObjectInputStream ois = new ObjectInputStream( new ByteArrayInputStream(serialised));

                try {
                    ajo_spec = (String) ois.readObject();
                }
                catch(Exception ex) {
                    throw new IOException("Errors reading AJO specification version from AJO consign form: "+ex);
                }

                try {
                    time_stamp = (java.util.Date) ois.readObject();
                }
                catch(Exception ex) {
                    throw new IOException("Errors reading date stamp from AJO consign form: "+ex);
                }

                deserialised = (AbstractJob) ois.readObject();

                // original.setId( deserialised.getId()); OK transported plain
                // original.setVsite( deserialised.getVsite()); OK transported plain
                // original.setEndorser( deserialised.getEndorser()); OK transported plain

                original.setSSO( deserialised.getSSO()); 
                original.setExtraInformation( deserialised.getExtraInformation()); 

                // Do not set fields of Idiomatics and so could get out
                // of sync here e.g what were references into the DAG
                // are no longer references, malicious changes etc

                // Now set the original's DAG
                Enumeration actions = deserialised.getActions();
                while(actions.hasMoreElements()) {
                    original.add((AbstractAction)actions.nextElement());
                }
                Enumeration deps = deserialised.getDependencies();
                while(deps.hasMoreElements()) {
                    original.add((Dependency)deps.nextElement());
                }

                // and now deserialise my children
                actions = original.getActions();
                while(actions.hasMoreElements()) {
                    AbstractAction child = (AbstractAction)actions.nextElement();
                    if(child instanceof AbstractJob) {
                        AJO blah = new AJO((AbstractJob)child); // What? Suppose its OK, converts in place
                        // convertFrom((AbstractJob)child);
                    }
                }

                // no longer need serialised form
                original.setConsignForm((byte[])null);

            }

            ajo = original;

        }
    }
}
//
// Change history:
//          30 Sep 2002 SvdB Convert Outcomes in byte form (Issue 4006)
//          30 Aug 2002 SvdB Convert Conditionals too
//          11 Apr 2002 SvdB Handle status history in outcomes, AJO time stamps and versions
//           7 Mar 2002 SvdB Need to copy the Outcome reason across
//           6 Apr 2001 SvdB Changes to handle endorser in AJO (comments only!)
//          23 Mar 2001 SvdB Convert sub-AJOs inside ActionGroups
//                           Added SignatureFactory as work around for JSSE Signature problem
//          13 Feb 2001 SvdB Clear serialised arrays on convertFrom
//          19 Dec 2000 SvdB Managed memory (for large AJOs)
//                           convertTo for Outcomes returns a copy
//          26 Jun 2000 SvdB added de/anonomise of Outcome
//           5 May 2000 SvdB to V3.0 
//          13 Apr 2000 SvdB Copied emails between User types 
//           9 Feb 2000 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
