package org.unicore.utility;

import java.io.InputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.io.EOFException;

/**
 * A sample implementation of a reader of a packetised
 * data stream as specified in the UPL.
 *
 * This reads an InputStream that has been split into variable
 * size packets. Each packet is preceeded by a 4 byte header
 * giving the size of the following packet. The final packet
 * in the stream is indicated by a 0 or negative size (absolute
 * value of this is the length of the final packet). Reading
 * past the end of a final packet generates an EOFException.
 *
 * This implementation assumes that the packet sizes are "large"
 * and so does not read past the end of a packet during one read
 * call (that is it may returns partial reads).
 *
 * @see org.unicore.utility.PacketisedOutputStream
 *
 * @author Sven van den Berghe (fecit) 
 *
 * @since AJO 3
 *
 * @version PacketisedInputStream.java 1.6 05/13/02
 *
 **/
public class PacketisedInputStream extends InputStream {

    private DataInputStream stream;

    private int left_in_packet;

    private boolean this_is_last;

    /**
     * Create a PacketisedInputStream based on the input stream.
     *
     **/
    public PacketisedInputStream(InputStream base_stream) {

        // Build a convenient way to read in the header bytes
        // on top of this stream. Most calls are onto the
        // read() methods which should be efficient.

        stream = new DataInputStream(base_stream); 

        left_in_packet = 0;
        this_is_last = false;

    }

    /**
     * Reads the next data byte from the stream. 
     * <p>
     * This conforms
     * to the InputStream contract and will block according to the
     * charactertistics of the underlying input stream(s). Packet
     * headers are stripped from the returned data.
     *
     **/
    public int read() throws IOException {
        try {
            startProcessing(1);
            int i = stream.read();
            endProcessing(1);
            return i;
        }
        catch(EmptyStreamException esex) {
            return -1;
        }
    }

    /**
     * Reads some bytes from the stream and stores them into b. 
     * <p>
     * This conforms
     * to the InputStream contract. Note that this contract allows partial reads
     * and this implementation will read only to the end of the current packet
     * regardless of the size of the array b. It will block according to the
     * charactertistics of the underlying input stream(s). Packet
     * headers are stripped from the returned data.
     *
     **/
    public int read(byte[] b) throws IOException {
        try {
            return endProcessing( stream.read( b, 0, startProcessing(b.length) ) );
        }
        catch(EmptyStreamException esex) {
            return -1;
        }
    }

    /**
     * Reads some bytes from the stream and stores them into b. 
     * <p>
     * This conforms
     * to the InputStream contract. Note that this contract allows partial reads
     * and this implementation will read only to the end of the current packet
     * regardless of len. It will block according to the
     * charactertistics of the underlying input stream(s). Packet
     * headers are stripped from the returned data.
     *
     **/
    public int read(byte[] b, int off, int len) throws IOException {
        try {
            return endProcessing( stream.read(b,off, startProcessing(len) ) );
        }
        catch(EmptyStreamException esex) {
            return -1;
        }
    }

    /**
     * Skips over and discards up to n data bytes from the stream
     * (and any headers found). 
     * <p>
     * Conforms to the InputStream contract.
     * This will skip over multiple headers if necessary.
     *
     **/
    public long skip(long n) throws IOException {

        if(n <= 0) return 0;

        long bytes_left_to_skip = n;

        try {
            while(bytes_left_to_skip > 0) {
                // How many bytes to skip in this call? Maximum
                // that will be skipped is the amount left in the 
                // packet so ...
                int to_read = startProcessing(left_in_packet);

                long skipped = stream.skip((long)to_read);  // this cast is upwards so safe
                if(skipped == 0) break;

                endProcessing((int)skipped);                // this cast is safe since to_read is an integer

                bytes_left_to_skip -= skipped;
            }
        }
        catch(EmptyStreamException esex) {}

        return n - bytes_left_to_skip;
    }

    /**
     * Return the minimum number of bytes available according
     * to the underlying input stream. 
     * <p>
     * This is the minimum of the number of
     * available bytes reported by the underlying stream and the 
     * number of bytes left in the current packet.
     *
     **/
    public int available() throws IOException {

        try {
            startProcessing(0);   // get a packet header if required
        }
        catch(EmptyStreamException esex) {
            throw new EOFException("PacketisedInputStream no more data available.");
        }

        // Have no idea how many packets are available
        // so return a conservative estimate
        int underlying = stream.available();
        if(underlying < left_in_packet) {
            return underlying;
        }
        else {
            return left_in_packet;
        }
    }

    /**
     * Close this InputStream, release any system resources
     * associated with the stream.
     * <p>
     * This calls the close() method on the underlying stream.
     *
     **/
    public void close() throws IOException {
        stream.close();
    }

    /**
     * Do processing after a read or skip
     * <p>
     * Input is the actual number of bytes read, which is 
     * passed to the output.
     *
     **/
    private int endProcessing(int bytes_read) throws IOException {
        
        if(bytes_read < -1) {
            throw new EOFException("PacketisedInputStream: unexpectedly at end of base stream");
        }

        left_in_packet -= bytes_read;

        if(left_in_packet <0) {
            throw new IOException("PacketisedInputStream: read over end of packet? <"+left_in_packet+">");
        }

        // DEBUG System.out.println("PacketisedInputStream endProcessing() called: "+bytes_read+" "+left_in_packet);

        return bytes_read;

    }

    
        

    /**
     * Do the processing at the start of a read or skip request.
     * <p>
     * Input is the maximum number of bytes to be read or skipped
     * in this request, output is the maximum available now.
     *
     **/
    private int startProcessing(int requested_bytes) throws IOException, EmptyStreamException  {

        // Do we need to read a packet header?

        // DEBUG System.out.println("PacketisedInputStream startProcessing() called: "+requested_bytes);

        if(left_in_packet==0) {
            // DEBUG System.out.println("PacketisedInputStream left_in_packet==0");

            if(this_is_last) {
                // Got to the end of the last packet
                // DEBUG System.out.println("PacketisedInputStream at end of last packet");
                throw new EmptyStreamException();
            }

            try {
                left_in_packet = stream.readInt();
                // DEBUG System.out.println("PacketisedInputStream left_in_packet reread to: "+left_in_packet);
            }
            catch(EOFException eofex) {
                throw new EOFException("PacketisedInputStream: unexpectedly at end of base stream in header read");
            }

            if(left_in_packet == 0) {
                // Asked to read but the next packet is a empty last one so can't do
                // DEBUG System.out.println("PacketisedInputStream this_is_last set to true left_in_packet==0");
                this_is_last = true;
                throw new EmptyStreamException();
            }

            if(left_in_packet < 0) { // Last packet with non-zero bytes
                this_is_last = true;
                left_in_packet = -1 * left_in_packet;
                // DEBUG System.out.println("PacketisedInputStream this_is_last set to true left_in_packet="+left_in_packet);
            }

        }

        // How many bytes do we read from the base stream?
        //
        // If there is no header due in the whole read, then
        // try to read all of them. Otherwise read just up
        // to the header.
        //

        // DEBUG System.out.println("PacketisedInputStream on exit "+left_in_packet);

        int bytes_to_read;
        if(requested_bytes <= left_in_packet) {
            bytes_to_read = requested_bytes;
        }
        else {
            bytes_to_read = left_in_packet;
        }
            
        return bytes_to_read;

    }

    private class EmptyStreamException extends Exception {
    }

}
//
// Change history:
//          19 Dec 2000 SvdB Set this_is_last when 0 size read
//          26 Sep 2000 SvdB Created
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
